@@ -42,19 +42,11 @@ struct plthook_arch_context {
 
 struct mcount_disasm_engine;
 struct mcount_dynamic_info;
+struct mcount_disasm_info;
 struct sym;
 
 int disasm_check_insns(struct mcount_disasm_engine *disasm,
-		       struct mcount_dynamic_info *mdi, struct sym *sym);
-
-struct dynamic_bad_symbol {
-	struct list_head	list;
-	struct sym		*sym;
-};
-
-struct dynamic_bad_symbol * find_bad_jump(struct mcount_dynamic_info *mdi,
-					  unsigned long addr);
-bool add_bad_jump(struct mcount_dynamic_info *mdi, unsigned long callsite,
-		  unsigned long target);
+		       struct mcount_dynamic_info *mdi,
+		       struct mcount_disasm_info *info);
 
 #endif /* MCOUNT_ARCH_H */
