@@ -12,7 +12,10 @@
 #include "utils/symbol.h"
 
 #define PAGE_SIZE  4096
+#define PAGE_ADDR(a)    ((void *)((a) & ~(PAGE_SIZE - 1)))
+#define PAGE_LEN(a, l)    (a + l - (unsigned long)PAGE_ADDR(a))
 #define XRAY_SECT  "xray_instr_map"
+#define MCOUNTLOC_SECT  "__mcount_loc"
 
 #define CALL_INSN_SIZE  5
 #define JMP_INSN_SIZE   6
@@ -32,15 +35,22 @@ struct xray_instr_map {
 
 enum mcount_x86_dynamic_type {
 	DYNAMIC_NONE,
+	DYNAMIC_PG,
 	DYNAMIC_FENTRY,
+	DYNAMIC_FENTRY_NOP,
 	DYNAMIC_XRAY,
 };
 
+static const char *adi_type_names[] = {
+	"none", "pg", "fentry", "fentry-nop", "xray",
+};
+
 struct arch_dynamic_info {
 	enum mcount_x86_dynamic_type	type;
 	struct xray_instr_map		*xrmap;
+	unsigned long			*mcount_loc;
 	unsigned			xrmap_count;
-	struct list_head		bad_targets;  /* for non-local jumps */
+	unsigned			nr_mcount_loc;
 };
 
 int mcount_setup_trampoline(struct mcount_dynamic_info *mdi)
@@ -76,7 +86,8 @@ int mcount_setup_trampoline(struct mcount_dynamic_info *mdi)
 			pr_err("failed to mmap trampoline for setup");
 	}
 
-	if (mprotect((void *)mdi->text_addr, mdi->text_size,
+	if (mprotect(PAGE_ADDR(mdi->text_addr), 
+			 PAGE_LEN(mdi->text_addr, mdi->text_size),
 		     PROT_READ | PROT_WRITE | PROT_EXEC)) {
 		pr_dbg("cannot setup trampoline due to protection: %m\n");
 		return -1;
@@ -93,7 +104,7 @@ int mcount_setup_trampoline(struct mcount_dynamic_info *mdi)
 		memcpy((void *)mdi->trampoline + 16 + sizeof(trampoline),
 		       &xray_exit_addr, sizeof(xray_exit_addr));
 	}
-	else if (adi->type == DYNAMIC_FENTRY) {
+	else if (adi->type == DYNAMIC_FENTRY_NOP) {
 		/* jmpq  *0x2(%rip)     # <fentry_addr> */
 		memcpy((void *)mdi->trampoline, trampoline, sizeof(trampoline));
 		memcpy((void *)mdi->trampoline + sizeof(trampoline),
@@ -114,7 +125,8 @@ int mcount_setup_trampoline(struct mcount_dynamic_info *mdi)
 
 void mcount_cleanup_trampoline(struct mcount_dynamic_info *mdi)
 {
-	if (mprotect((void *)mdi->text_addr, mdi->text_size, PROT_EXEC))
+	if (mprotect(PAGE_ADDR(mdi->text_addr), 
+			 PAGE_LEN(mdi->text_addr, mdi->text_size), PROT_EXEC))
 		pr_err("cannot restore trampoline due to protection");
 }
 
@@ -145,20 +157,40 @@ static void read_xray_map(struct arch_dynamic_info *adi,
 	}
 }
 
+static void read_mcount_loc(struct arch_dynamic_info *adi,
+			    struct uftrace_elf_data *elf,
+			    struct uftrace_elf_iter *iter,
+			    unsigned long offset)
+{
+	typeof(iter->shdr) *shdr = &iter->shdr;
+
+	adi->nr_mcount_loc = shdr->sh_size / sizeof(long);
+	adi->mcount_loc = xmalloc(shdr->sh_size);
+
+	elf_get_secdata(elf, iter);
+	elf_read_secdata(elf, iter, 0, adi->mcount_loc, shdr->sh_size);
+
+	/* symbol has relative address, fix it to match each other */
+	if (elf->ehdr.e_type == ET_EXEC) {
+		unsigned i;
+
+		for (i = 0; i < adi->nr_mcount_loc; i++) {
+			adi->mcount_loc[i] -= offset;
+		}
+	}
+}
+
 void mcount_arch_find_module(struct mcount_dynamic_info *mdi,
 			     struct symtab *symtab)
 {
 	struct uftrace_elf_data elf;
 	struct uftrace_elf_iter iter;
 	struct arch_dynamic_info *adi;
-	const char *adi_type_names[] = { "none", "fentry", "xray" };
-	unsigned char fentry_patt1[] = { 0x67, 0x0f, 0x1f, 0x04, 0x00 };
-	unsigned char fentry_patt2[] = { 0x0f, 0x1f, 0x44, 0x00, 0x00 };
-	int num_check = 5;
+	unsigned char fentry_nop_patt1[] = { 0x67, 0x0f, 0x1f, 0x04, 0x00 };
+	unsigned char fentry_nop_patt2[] = { 0x0f, 0x1f, 0x44, 0x00, 0x00 };
 	unsigned i = 0;
 
 	adi = xzalloc(sizeof(*adi));  /* DYNAMIC_NONE */
-	INIT_LIST_HEAD(&adi->bad_targets);
 
 	if (elf_init(mdi->map->libname, &elf) < 0)
 		goto out;
@@ -171,6 +203,11 @@ void mcount_arch_find_module(struct mcount_dynamic_info *mdi,
 			read_xray_map(adi, &elf, &iter, mdi->base_addr);
 			goto out;
 		}
+
+		if (!strcmp(shstr, MCOUNTLOC_SECT)) {
+			read_mcount_loc(adi, &elf, &iter, mdi->base_addr);
+			/* still needs to check pg or fentry */
+		}
 	}
 
 	/* check first few functions have fentry signature */
@@ -186,19 +223,27 @@ void mcount_arch_find_module(struct mcount_dynamic_info *mdi,
 			continue;
 
 		/* only support calls to __fentry__ at the beginning */
-		if (!memcmp(code_addr, fentry_patt1, CALL_INSN_SIZE) ||
-		    !memcmp(code_addr, fentry_patt2, CALL_INSN_SIZE)) {
-			adi->type = DYNAMIC_FENTRY;
-			break;
+		if (!memcmp(code_addr, fentry_nop_patt1, CALL_INSN_SIZE) ||
+		    !memcmp(code_addr, fentry_nop_patt2, CALL_INSN_SIZE)) {
+			adi->type = DYNAMIC_FENTRY_NOP;
+			goto out;
 		}
+	}
 
-		if (num_check-- == 0)
-			break;
+	switch (check_trace_functions(mdi->map->libname)) {
+	case TRACE_MCOUNT:
+		adi->type = DYNAMIC_PG;
+		break;
+	case TRACE_FENTRY:
+		adi->type = DYNAMIC_FENTRY;
+		break;
+	default:
+		break;
 	}
 
 out:
-	pr_dbg("dynamic patch type: %d (%s)\n", adi->type,
-	       adi_type_names[adi->type]);
+	pr_dbg("dynamic patch type: %s: %d (%s)\n", basename(mdi->map->libname),
+	       adi->type, adi_type_names[adi->type]);
 
 	mdi->arch = adi;
 	elf_finish(&elf);
@@ -411,68 +456,24 @@ static void patch_code(struct mcount_dynamic_info *mdi,
 				origin_code_addr + origin_code_size);
 }
 
-struct dynamic_bad_symbol * find_bad_jump(struct mcount_dynamic_info *mdi,
-					  unsigned long addr)
-{
-	struct sym *sym;
-	struct arch_dynamic_info *adi = mdi->arch;
-	struct dynamic_bad_symbol *badsym;
-
-	sym = find_sym(&mdi->map->mod->symtab, addr - mdi->map->start);
-	if (sym == NULL)
-		return NULL;
-
-	list_for_each_entry(badsym, &adi->bad_targets, list) {
-		if (badsym->sym == sym)
-			return badsym;
-	}
-
-	return NULL;
-}
-
-bool add_bad_jump(struct mcount_dynamic_info *mdi, unsigned long callsite,
-		  unsigned long target)
-{
-	struct sym *sym;
-	struct arch_dynamic_info *adi = mdi->arch;
-	struct dynamic_bad_symbol *badsym;
-
-	if (find_bad_jump(mdi, target))
-		return true;
-
-	sym = find_sym(&mdi->map->mod->symtab, target - mdi->map->start);
-	if (sym == NULL)
-		return true;
-
-	/* only care about jumps to the middle of a function */
-	if (sym->addr + mdi->map->start == target)
-		return false;
-
-	pr_dbg2("bad jump: %s:%lx to %lx\n", sym ? sym->name : "<unknown>",
-		callsite - mdi->map->start, target - mdi->map->start);
-
-	badsym = xmalloc(sizeof(*badsym));
-	badsym->sym = sym;
-
-	list_add_tail(&badsym->list, &adi->bad_targets);
-	return true;
-}
-
 static int patch_normal_func(struct mcount_dynamic_info *mdi, struct sym *sym,
 			     struct mcount_disasm_engine *disasm)
 {
-	int instr_size;
 	uint8_t jmp_insn[14] = { 0xff, 0x25, };
 	uint64_t jmp_target;
 	struct mcount_orig_insn *orig;
-	uint64_t sym_addr = sym->addr + mdi->map->start;
+	struct mcount_disasm_info info = {
+		.sym  = sym,
+		.addr = mdi->map->start + sym->addr,
+	};
+	int state;
 
-	instr_size = disasm_check_insns(disasm, mdi, sym);
-	if (instr_size < CALL_INSN_SIZE)
-		return instr_size;
+	state = disasm_check_insns(disasm, mdi, &info);
+	if (state != INSTRUMENT_SUCCESS)
+		return state;
 
 	pr_dbg2("patch normal func: %s (patch size: %d)\n",
-		sym->name, instr_size);
+		sym->name, info.orig_size);
 
 	/*
 	 *  stored origin instruction block:
@@ -484,17 +485,84 @@ static int patch_normal_func(struct mcount_dynamic_info *mdi, struct sym *sym,
 	 *  | [Return   address] |
 	 *  ----------------------
 	 */
-	jmp_target = sym_addr + instr_size;
+	jmp_target = info.addr + info.orig_size;
 	memcpy(jmp_insn + JMP_INSN_SIZE, &jmp_target, sizeof(jmp_target));
-	orig = mcount_save_code(sym_addr , instr_size,
-				jmp_insn, sizeof(jmp_insn));
+
+	if (info.has_jump)
+		orig = mcount_save_code(&info, jmp_insn, 0);
+	else
+		orig = mcount_save_code(&info, jmp_insn, sizeof(jmp_insn));
+
 	/* make sure orig->addr same as when called from __dentry__ */
 	orig->addr += CALL_INSN_SIZE;
+	patch_code(mdi, info.addr, info.orig_size);
 
-	patch_code(mdi, sym_addr, instr_size);
 	return INSTRUMENT_SUCCESS;
 }
 
+static int unpatch_func(uint8_t *insn, char *name)
+{
+	uint8_t nop5[] = { 0x0f, 0x1f, 0x44, 0x00, 0x00 };
+	uint8_t nop6[] = { 0x66, 0x0f, 0x1f, 0x44, 0x00, 0x00 };
+	uint8_t *nop_insn;
+	size_t nop_size;
+
+	if (*insn == 0xe8) {
+		nop_insn = nop5;
+		nop_size = sizeof(nop5);
+	}
+	else if (insn[0] == 0xff && insn[1] == 0x15) {
+		nop_insn = nop6;
+		nop_size = sizeof(nop6);
+	}
+	else {
+		return INSTRUMENT_SKIPPED;
+	}
+
+	pr_dbg3("unpatch fentry: %s\n", name);
+	memcpy(insn, nop_insn, nop_size);
+	__builtin___clear_cache(insn, insn + nop_size);
+
+	return INSTRUMENT_SUCCESS;
+}
+
+static int unpatch_fentry_func(struct mcount_dynamic_info *mdi, struct sym *sym)
+{
+	uint64_t sym_addr = sym->addr + mdi->map->start;
+
+	return unpatch_func((void *)sym_addr, sym->name);
+}
+
+static int cmp_loc(const void *a, const void *b)
+{
+	const struct sym *sym = a;
+	uintptr_t loc = *(uintptr_t *)b;
+
+	if (sym->addr <= loc && loc < sym->addr + sym->size)
+		return 0;
+
+	return sym->addr > loc ? 1 : -1;
+}
+
+static int unpatch_mcount_func(struct mcount_dynamic_info *mdi, struct sym *sym)
+{
+	struct arch_dynamic_info *adi = mdi->arch;
+
+	uintptr_t *loc;
+
+	if (adi->nr_mcount_loc != 0) {
+		loc = bsearch(sym, adi->mcount_loc, adi->nr_mcount_loc,
+			       sizeof(*adi->mcount_loc), cmp_loc);
+
+		if (loc != NULL) {
+			uint8_t *insn = (uint8_t*) *loc;
+			return unpatch_func(insn + mdi->map->start, sym->name);
+		}
+	}
+
+	return INSTRUMENT_SKIPPED;
+}
+
 int mcount_patch_func(struct mcount_dynamic_info *mdi, struct sym *sym,
 		      struct mcount_disasm_engine *disasm,
 		      unsigned min_size)
@@ -513,7 +581,7 @@ int mcount_patch_func(struct mcount_dynamic_info *mdi, struct sym *sym,
 		result = patch_xray_func(mdi, sym);
 		break;
 
-	case DYNAMIC_FENTRY:
+	case DYNAMIC_FENTRY_NOP:
 		result = patch_fentry_func(mdi, sym);
 		break;
 
@@ -527,41 +595,50 @@ int mcount_patch_func(struct mcount_dynamic_info *mdi, struct sym *sym,
 	return result;
 }
 
-#define INSN_CHECK_LEN  16
+int mcount_unpatch_func(struct mcount_dynamic_info *mdi, struct sym *sym,
+			struct mcount_disasm_engine *disasm)
+{
+	struct arch_dynamic_info *adi = mdi->arch;
+	int result = INSTRUMENT_SKIPPED;
+
+	switch (adi->type) {
+	case DYNAMIC_FENTRY:
+		result = unpatch_fentry_func(mdi, sym);
+		break;
+
+	case DYNAMIC_PG:
+		result = unpatch_mcount_func(mdi, sym);
+		break;
+
+	default:
+		break;
+	}
+	return result;
+}
 
 static void revert_normal_func(struct mcount_dynamic_info *mdi, struct sym *sym,
 			       struct mcount_disasm_engine *disasm)
 {
-	uint8_t jmp_insn[6] = { 0xff, 0x25, };
 	void *addr = (void *)(uintptr_t)sym->addr + mdi->map->start;
-	void *saved_insn;
-	int i;
+	struct mcount_orig_insn *moi;
 
-	saved_insn = mcount_find_code((uintptr_t)addr + CALL_INSN_SIZE);
-	if (saved_insn == NULL)
+	moi = mcount_find_insn((uintptr_t)addr + CALL_INSN_SIZE);
+	if (moi == NULL)
 		return;
 
-	/* we don't the original copy size, find the jmp insn instead */
-	for (i = CALL_INSN_SIZE; i < INSN_CHECK_LEN; i++) {
-		if (!memcmp(saved_insn + i, jmp_insn, sizeof(jmp_insn)))
-			break;
-	}
-
-	if (i == INSN_CHECK_LEN)
-		pr_err_ns("cannot find original insn length\n");
-
-	memcpy(addr, saved_insn, i);
-	__builtin___clear_cache(addr, addr + i);
+	memcpy(addr, moi->orig, moi->orig_size);
+	__builtin___clear_cache(addr, addr + moi->orig_size);
 }
 
 void mcount_arch_dynamic_recover(struct mcount_dynamic_info *mdi,
 				 struct mcount_disasm_engine *disasm)
 {
-	struct arch_dynamic_info *adi = mdi->arch;
 	struct dynamic_bad_symbol *badsym, *tmp;
 
-	list_for_each_entry_safe(badsym, tmp, &adi->bad_targets, list) {
-		revert_normal_func(mdi, badsym->sym, disasm);
+	list_for_each_entry_safe(badsym, tmp, &mdi->bad_syms, list) {
+		if (!badsym->reverted)
+			revert_normal_func(mdi, badsym->sym, disasm);
+
 		list_del(&badsym->list);
 		free(badsym);
 	}
