@@ -48,7 +48,7 @@ struct uftrace_dump_ops {
 	void (*kernel_event)(struct uftrace_dump_ops *ops,
 			     struct uftrace_kernel_reader *kernel, int cpu,
 			     struct uftrace_record *frs);
-	/* thius is called when there's a lost record (usually in kernel) */
+	/* this is called when there's a lost record (usually in kernel) */
 	void (*lost)(struct uftrace_dump_ops *ops,
 		     uint64_t time, int tid, int losts);
 	/* this is called when a perf data (for each cpu) starts */
@@ -823,6 +823,7 @@ static void dump_raw_perf_event(struct uftrace_dump_ops *ops,
 	free(evt_name);
 }
 
+/* chrome support */
 static void dump_chrome_header(struct uftrace_dump_ops *ops,
 				struct uftrace_data *handle,
 				struct opts *opts)
@@ -860,7 +861,7 @@ static void dump_chrome_task_rstack(struct uftrace_dump_ops *ops,
 	char ph;
 	char spec_buf[1024];
 	struct uftrace_record *frs = task->rstack;
-	enum argspec_string_bits str_mode = NEEDS_ESCAPE;
+	enum argspec_string_bits str_mode = NEEDS_JSON;
 	struct uftrace_chrome_dump *chrome = container_of(ops, typeof(*chrome), ops);
 	bool is_process = task->t->pid == task->tid;
 	int rec_type = frs->type;
@@ -1029,10 +1030,10 @@ static void adjust_fg_time(struct uftrace_task_graph *tg, void *arg)
 {
 	struct uftrace_dump_ops *ops = arg;
 	struct uftrace_flame_dump *flame = container_of(ops, typeof(*flame), ops);
-	struct fstack *fstack = &tg->task->func_stack[tg->task->stack_count];
 	struct uftrace_graph_node *node = tg->node;
-	uint64_t curr_time = fstack->total_time;
-	uint64_t sample_time = flame->sample_time;
+	struct fstack *fstack;
+	uint64_t curr_time;
+	uint64_t sample_time;
 	uint64_t accounted_time;
 
 	if (flame->sample_time == 0)
@@ -1041,6 +1042,13 @@ static void adjust_fg_time(struct uftrace_task_graph *tg, void *arg)
 	if (tg->node->parent == NULL)
 		return;
 
+	fstack = fstack_get(tg->task, tg->task->stack_count);
+	if (fstack == NULL)
+		return;
+
+	curr_time = fstack->total_time;
+	sample_time = flame->sample_time;
+
 	/*
 	 * it needs to track the child time separately
 	 * since child time not accounted due to sample time
@@ -1058,7 +1066,7 @@ static void adjust_fg_time(struct uftrace_task_graph *tg, void *arg)
 	 * so 10 samples should be shown, but after accounting
 	 * foo and bar (4 samples each), its time would be
 	 * 10.567 - 4.789 - 4.987 = 0.791 so no samples for main.
-	 * But it acctually needs to get 2 samples.
+	 * But it actually needs to get 2 samples.
 	 *
 	 * So add the accounted child time only, not real time.
 	 */
@@ -1160,7 +1168,7 @@ static void dump_flame_footer(struct uftrace_dump_ops *ops,
 	graph_remove_task();
 }
 
-/* to graphviz support */
+/* graphviz support */
 static struct uftrace_graph graphviz_graph = {
 	.root.head     = LIST_HEAD_INIT(graphviz_graph.root.head),
 	.special_nodes = LIST_HEAD_INIT(graphviz_graph.special_nodes),
@@ -1508,7 +1516,7 @@ static void do_dump_replay(struct uftrace_dump_ops *ops, struct opts *opts,
 			continue;
 
 		if (prev_time > frs->time)
-			ops->inverted_time(ops, task);
+			call_if_nonull(ops->inverted_time, ops, task);
 		prev_time = frs->time;
 
 
@@ -1536,7 +1544,9 @@ static void do_dump_replay(struct uftrace_dump_ops *ops, struct opts *opts,
 			struct fstack *fstack;
 			struct uftrace_session *fsess = handle->sessions.first;
 
-			fstack = &task->func_stack[task->stack_count];
+			fstack = fstack_get(task, task->stack_count);
+			if (fstack == NULL)
+				continue;
 
 			if (fstack->addr == 0)
 				continue;
