@@ -14,6 +14,7 @@
 #include "utils/graph.h"
 
 static LIST_HEAD(output_fields);
+static LIST_HEAD(output_task_fields);
 
 struct graph_backtrace {
 	struct list_head list;
@@ -67,7 +68,7 @@ static void print_addr(struct field_data *fd)
 	/* uftrace records (truncated) 48-bit addresses */
 	int width = sizeof(long) == 4 ? 8 : 12;
 
-	pr_out("%*lx", width, node->addr);
+	pr_out("%*"PRIx64, width, effective_addr(node->addr));
 }
 
 static struct display_field field_total_time= {
@@ -105,6 +106,61 @@ static struct display_field field_addr = {
 	.list    = LIST_HEAD_INIT(field_addr.list),
 };
 
+static void print_task_total_time(struct field_data *fd)
+{
+	struct uftrace_task *node = fd->arg;
+	uint64_t d;
+
+	d = node->time.run;
+
+	print_time_unit(d);
+}
+
+static void print_task_self_time(struct field_data *fd)
+{
+	struct uftrace_task *node = fd->arg;
+	uint64_t d;
+
+	d = node->time.run - node->time.idle;
+
+	print_time_unit(d);
+}
+
+static void print_task_tid(struct field_data *fd)
+{
+	struct uftrace_task *task = fd->arg;
+	pr_out("[%6d]", task->tid);
+}
+
+static struct display_field field_task_total_time = {
+	.id      = GRAPH_F_TASK_TOTAL_TIME,
+	.name    = "total-time",
+	.alias   = "total",
+	.header  = "TOTAL TIME",
+	.length  = 10,
+	.print   = print_task_total_time,
+	.list    = LIST_HEAD_INIT(field_task_total_time.list),
+};
+
+static struct display_field field_task_self_time = {
+	.id      = GRAPH_F_TASK_SELF_TIME,
+	.name    = "self-time",
+	.alias   = "self",
+	.header  = " SELF TIME",
+	.length  = 10,
+	.print   = print_task_self_time,
+	.list    = LIST_HEAD_INIT(field_task_self_time.list),
+};
+
+static struct display_field field_task_tid = {
+	.id      = GRAPH_F_TASK_TID,
+	.name    = "tid",
+	.header  = "   TID  ",
+	.length  = 8,
+	.print   = print_task_tid,
+	.list    = LIST_HEAD_INIT(field_task_tid.list),
+};
+
 /* index of this table should be matched to display_field_id */
 static struct display_field *field_table[] = {
 	&field_total_time,
@@ -112,11 +168,25 @@ static struct display_field *field_table[] = {
 	&field_addr,
 };
 
+/* index of this task table should be matched to display_field_id */
+static struct display_field *field_task_table[] = {
+	&field_task_total_time,
+	&field_task_self_time,
+	&field_task_tid,
+};
+
 static void setup_default_field(struct list_head *fields, struct opts *opts)
 {
 	add_field(fields, field_table[GRAPH_F_TOTAL_TIME]);
 }
 
+static void setup_default_task_field(struct list_head *fields, struct opts *opts)
+{
+	add_field(fields, field_task_table[GRAPH_F_TASK_TOTAL_TIME]);
+	add_field(fields, field_task_table[GRAPH_F_TASK_SELF_TIME]);
+	add_field(fields, field_task_table[GRAPH_F_TASK_TID]);
+}
+
 static void print_field(struct uftrace_graph_node *node)
 {
 	struct field_data fd = {
@@ -127,6 +197,16 @@ static void print_field(struct uftrace_graph_node *node)
 		pr_out(" : ");
 }
 
+static void print_task_field(struct uftrace_task *node)
+{
+	struct field_data fd = {
+		.arg = node,
+	};
+
+	if (print_field_data(&output_task_fields, &fd, 2))
+		pr_out(" : ");
+}
+
 static int create_graph(struct uftrace_session *sess, void *func)
 {
 	struct session_graph *graph = xzalloc(sizeof(*graph));
@@ -230,8 +310,14 @@ static int save_backtrace_addr(struct task_graph *tg)
 	if (len == 0)
 		return 0;
 
-	for (i = len - 1; i >= 0; i--)
-		addrs[i] = task->func_stack[i + skip].addr;
+	for (i = len - 1; i >= 0; i--) {
+		struct fstack *fstack = fstack_get(task, i + skip);
+
+		if (fstack != NULL)
+			addrs[i] = fstack->addr;
+		else
+			addrs[i] = 0;
+	}
 
 	list_for_each_entry(bt, &graph->bt_list, list) {
 		if (len == bt->len &&
@@ -258,9 +344,9 @@ found:
 static void save_backtrace_time(struct task_graph *tg)
 {
 	struct uftrace_task_reader *task = tg->utg.task;
-	struct fstack *fstack = &task->func_stack[task->stack_count];
+	struct fstack *fstack = fstack_get(task, task->stack_count);
 
-	if (tg->bt_curr)
+	if (tg->bt_curr != NULL && fstack != NULL)
 		tg->bt_curr->time += fstack->total_time;
 
 	tg->bt_curr = NULL;
@@ -296,7 +382,7 @@ static int print_backtrace(struct session_graph *graph)
 
 static int start_graph(struct task_graph *tg)
 {
-	if (!tg->enabled++) {
+	if (tg->utg.graph && !tg->enabled++) {
 		save_backtrace_addr(tg);
 
 		pr_dbg("start graph for task %d\n", tg->utg.task->tid);
@@ -421,7 +507,7 @@ static int print_graph(struct session_graph *graph, struct opts *opts)
 
 	if (graph->ug.root.time || graph->ug.root.nr_edges) {
 		pr_out("========== FUNCTION CALL GRAPH ==========\n");
-		print_header(&output_fields, "# ", 2);
+		print_header(&output_fields, "# ", "FUNCTION", 2);
 		indent_mask = xcalloc(opts->max_stack, sizeof(*indent_mask));
 		print_graph_node(&graph->ug, &graph->ug.root, indent_mask, 0,
 				 graph->ug.root.nr_edges > 1);
@@ -440,6 +526,8 @@ static void build_graph_node(struct opts *opts,
 	char *name;
 
 	tg = get_task_graph(task, time, addr);
+	if (unlikely(tg->utg.graph == NULL))
+		return;
 
 	sym = find_symtabs(&tg->utg.graph->sess->symtabs, addr);
 	if (sym == NULL)
@@ -521,9 +609,9 @@ static void build_graph(struct opts *opts, struct uftrace_data *handle,
 			while (task->stack_count >= task->user_stack_count) {
 				struct fstack *fstack;
 
-				fstack = &task->func_stack[task->stack_count];
+				fstack = fstack_get(task, task->stack_count);
 
-				if (fstack_enabled && fstack->valid &&
+				if (fstack_enabled && fstack && fstack->valid &&
 				    !(fstack->flags & FSTACK_FL_NORECORD)) {
 					build_graph_node(opts, task, prev_time,
 							 fstack->addr,
@@ -553,9 +641,6 @@ static void build_graph(struct opts *opts, struct uftrace_data *handle,
 		}
 		prev_time = frs->time;
 
-		if (task->stack_count >= opts->max_stack)
-			continue;
-
 		build_graph_node(opts, task, frs->time, addr, frs->type, func);
 	}
 
@@ -575,7 +660,9 @@ static void build_graph(struct opts *opts, struct uftrace_data *handle,
 			last_time = handle->time_range.stop;
 
 		while (--task->stack_count >= 0) {
-			fstack = &task->func_stack[task->stack_count];
+			fstack = fstack_get(task, task->stack_count);
+			if (fstack == NULL)
+				continue;
 
 			if (fstack->addr == 0)
 				continue;
@@ -654,6 +741,217 @@ static void synthesize_depth_trigger(struct opts *opts,
 		 func, ffd.found ? "" : "kernel,", opts->depth);
 }
 
+static void reset_task_runtime(struct uftrace_data *handle)
+{
+	struct uftrace_task *t;
+	struct rb_node *n = rb_first(&handle->sessions.tasks);
+
+	while (n != NULL) {
+		t = rb_entry(n, struct uftrace_task, node);
+		n = rb_next(n);
+
+		t->time.run = 0;
+		t->time.idle = 0;
+		t->time.stamp = 0;
+	}
+}
+
+static void graph_build_task(struct opts *opts, struct uftrace_data *handle)
+{
+	struct uftrace_task_reader *task;
+	struct uftrace_task *t;
+	int i;
+
+	/*
+	 * we need to know entire runtime, so not apply time filter now
+	 * and it will be filtered when it's printed.
+	 */
+	handle->time_filter = 0;
+	reset_task_runtime(handle);
+
+	while (!read_rstack(handle, &task) && !uftrace_done) {
+		struct uftrace_record *frs = task->rstack;
+
+		if (!fstack_check_opts(task, opts))
+			continue;
+
+		if (!fstack_check_filter(task))
+			continue;
+
+		if (task->timestamp == 0)
+			task->timestamp = frs->time;
+		task->timestamp_last = frs->time;
+
+		t = task->t;
+
+		if (frs->type == UFTRACE_EVENT) {
+			switch (frs->addr) {
+			case EVENT_ID_PERF_SCHED_OUT:
+				t->time.stamp = frs->time;
+				break;
+			case EVENT_ID_PERF_SCHED_IN:
+				if (t->time.stamp)
+					t->time.idle += frs->time - t->time.stamp;
+				t->time.stamp = 0;
+				break;
+			}
+		}
+	}
+
+	/* update task time if some records were missing */
+	for (i = 0; i < handle->nr_tasks; i++) {
+		task = &handle->tasks[i];
+		t = task->t;
+
+		/* update idle time if last SCHED_IN event was missing */
+		if (t->time.stamp)
+			t->time.idle += task->timestamp_last - t->time.stamp;
+
+		t->time.run = task->timestamp_last - task->timestamp;
+	}
+	handle->time_filter = opts->threshold;
+}
+
+/* returns true if any of child has more runtime than the filter */
+static bool check_time_filter(struct uftrace_task *task, struct opts *opts)
+{
+	struct uftrace_task *child;
+
+	list_for_each_entry(child, &task->children, siblings) {
+		if (child->time.run >= opts->threshold)
+			return true;
+		if (check_time_filter(child, opts))
+			return true;
+	}
+	return false;
+}
+
+static bool is_last_child(struct uftrace_task *task,
+			  struct uftrace_task *parent,
+			  struct opts *opts)
+{
+	if (list_is_singular(&parent->children) ||
+	    parent->children.prev == &task->siblings)
+		return true;
+
+	/* any sibling satisfies the time filter? */
+	list_for_each_entry_continue(task, &parent->children, siblings) {
+		if (task->time.run >= opts->threshold ||
+		    check_time_filter(task, opts))
+			return false;
+	}
+	return true;
+}
+
+static bool print_task_node(struct uftrace_task *task,
+			    struct uftrace_task *parent,
+			    bool *indent_mask, int indent,
+			    struct opts *opts)
+{
+	char *name = task->comm;
+	struct uftrace_task *child;
+	int orig_indent = indent;
+	bool blank = false;
+
+	if (uftrace_done)
+		return false;
+
+	print_task_field(task);
+	pr_indent(indent_mask, indent, true);
+	if (parent && parent->pid == task->pid) {
+		/* print thread name in green color */
+		pr_green("%s\n", name);
+	}
+	else {
+		/* print process name */
+		pr_out("%s\n", name);
+	}
+
+	if (list_empty(&task->children) || !check_time_filter(task, opts))
+		return false;
+
+	/* clear parent indent mask at the last node */
+	if (parent && is_last_child(task, parent, opts)) {
+		int parent_indent = orig_indent - 1;
+
+		if (task->pid != parent->pid)
+			parent_indent--;
+
+		indent_mask[parent_indent] = false;
+	}
+
+	list_for_each_entry(child, &task->children, siblings) {
+		/*
+		 * Filter out if total time is less than time-filter.
+		 * Note that child might live longer than parent.
+		 * In that case we should print the parent even if it's
+		 * shorter than the time filter to show a correct tree.
+		 */
+		if (opts->threshold > child->time.run &&
+		    !check_time_filter(child, opts))
+			continue;
+
+		indent = orig_indent;
+
+		indent_mask[indent++] = true;
+		if (child->pid != task->pid) {
+			/* print blank line before forked child */
+			blank = true;
+			indent++;
+		}
+
+		if (blank) {
+			/* print blank line between siblings */
+			if (print_empty_field(&output_task_fields, 2))
+				pr_out(" : ");
+			pr_indent(indent_mask, indent, false);
+			pr_out("\n");
+
+			blank = false;
+		}
+
+		blank |= print_task_node(child, task, indent_mask, indent, opts);
+
+		if (&child->siblings != task->children.prev &&
+		    child->pid != task->pid) {
+			/* print blank line after forked child */
+			blank = true;
+		}
+	}
+	indent_mask[orig_indent] = false;
+	return blank;
+}
+
+static int graph_print_task(struct uftrace_data *handle, struct opts *opts)
+{
+	bool *indent_mask;
+	struct uftrace_task *task;
+
+	if (uftrace_done)
+		return 0;
+
+	if (handle->nr_tasks <= 0)
+		return 0;
+
+	task = handle->sessions.first_task;
+
+	setup_field(&output_task_fields, opts, &setup_default_task_field,
+		    field_task_table, ARRAY_SIZE(field_task_table));
+
+	pr_out("========== TASK GRAPH ==========\n");
+	print_header(&output_task_fields, "# ", "TASK NAME", 2);
+
+	indent_mask = xcalloc(handle->nr_tasks, sizeof(*indent_mask));
+
+	/* filter out if total time is less than time-filter */
+	if (opts->threshold <= task->time.run)
+		print_task_node(task, NULL, indent_mask, 0, opts);
+
+	free(indent_mask);
+	pr_out("\n");
+	return 1;
+}
+
 int command_graph(int argc, char *argv[], struct opts *opts)
 {
 	int ret;
@@ -689,6 +987,12 @@ int command_graph(int argc, char *argv[], struct opts *opts)
 
 	fstack_setup_filters(opts, &handle);
 
+	if (opts->show_task) {
+		graph_build_task(opts, &handle);
+		graph_print_task(&handle, opts);
+		goto out;
+	}
+
 	build_graph(opts, &handle, func);
 
 	graph = graph_list;
@@ -717,6 +1021,7 @@ int command_graph(int argc, char *argv[], struct opts *opts)
 	}
 	graph_remove_task();
 
+out:
 	close_data_file(opts, &handle);
 
 	return 0;
