@@ -209,7 +209,9 @@ static int fill_cmdline(void *arg)
 	struct fill_handler_arg *fha = arg;
 	char *buf = fha->buf;
 	FILE *fp;
-	int ret, i;
+	int err;
+	int ret;
+	int i;
 	char *p;
 
 	fp = fopen("/proc/self/cmdline", "r");
@@ -217,18 +219,21 @@ static int fill_cmdline(void *arg)
 		return -1;
 
 	ret = fread(buf, 1, sizeof(fha->buf), fp);
+	err = ferror(fp);
 	fclose(fp);
 
-	if (ret < 0)
-		return ret;
+	if (!ret && err)
+		return -1;
 
 	/* cmdline separated by NUL character - convert to space */
 	for (i = 0, p = buf; i < ret; i++, p++) {
-		if (*p == '\0')
+		if (*p == '\0' || *p == '\n')
 			*p = ' ';
 	}
 
-	p = strquote(buf, &ret);
+	buf[sizeof(fha->buf) - 1] = '\0';
+
+	p = json_quote(buf, &ret);
 	p[ret - 1] = '\n';
 
 	if ((write(fha->fd, "cmdline:", 8) < 8) ||
@@ -967,6 +972,10 @@ void clear_uftrace_info(struct uftrace_info *info)
 	free(info->record_date);
 	free(info->elapsed_time);
 	free(info->uftrace_version);
+	free(info->retspec);
+	free(info->autoarg);
+	free(info->autoret);
+	free(info->autoenum);
 }
 
 static void print_info(void *unused, const char *fmt, ...)
@@ -1139,6 +1148,64 @@ void process_uftrace_info(struct uftrace_data *handle, struct opts *opts,
 	process(data, "\n");
 }
 
+static void print_task(struct uftrace_data *handle, struct uftrace_task *t)
+{
+	char flags[6] = "     ";
+	struct stat stbuf;
+	char *filename = NULL;
+	struct uftrace_sess_ref *sref = &t->sref;
+
+	xasprintf(&filename, "%s/%d.dat", handle->dirname, t->tid);
+	if (stat(filename, &stbuf) < 0)
+		stbuf.st_size = 0;
+
+	if (t->tid == t->pid)
+		flags[0] = 'F';  /* FORK */
+	while (sref != NULL) {
+		if (sref->sess->tid == t->tid) {
+			flags[1] = 'S';  /* SESSION */
+			break;
+		}
+
+		sref = sref->next;
+	}
+
+	/* TIMESTAMP */
+	pr_out(" %13lu.%09lu ", t->time.stamp / NSEC_PER_SEC,
+	       t->time.stamp % NSEC_PER_SEC);
+	/* FLAGS  TID  COMM */
+	pr_out(" %s  [%6d]  %-16s ", flags, t->tid, t->comm);
+	/* DATA SIZE */
+	if (stbuf.st_size) {
+		uint64_t size_kb = stbuf.st_size / 1024;
+		uint64_t size_rem = size_kb % 1024;
+
+		pr_out(" %5"PRIu64".%03"PRIu64" MB\n", size_kb / 1024,
+		       size_rem >= 1000 ? 999 : size_rem);
+	}
+	else
+		pr_out("\n");
+
+	free(filename);
+}
+
+static void print_task_info(struct uftrace_data *handle)
+{
+	struct rb_node *n;
+	struct uftrace_task *t;
+
+	pr_out("#%23s  %5s  %8s  %-16s  %s\n",
+	       "TIMESTAMP     ", "FLAGS", "  TID  ", "TASK", "DATA SIZE");
+
+	n = rb_first(&handle->sessions.tasks);
+	while (n != NULL) {
+		t = rb_entry(n, struct uftrace_task, node);
+		n = rb_next(n);
+
+		print_task(handle, t);
+	}
+}
+
 int command_info(int argc, char *argv[], struct opts *opts)
 {
 	int ret;
@@ -1172,7 +1239,21 @@ int command_info(int argc, char *argv[], struct opts *opts)
 		goto out;
 	}
 
-	fstack_setup_filters(opts, &handle);
+	fstack_setup_task(opts->tid, &handle);
+	if (opts->show_task) {
+		/* ignore errors */
+		read_task_txt_file(&handle.sessions, opts->dirname,
+				   false, false, false);
+
+		if (handle.hdr.feat_mask & PERF_EVENT) {
+			if (setup_perf_data(&handle) == 0)
+				update_perf_task_comm(&handle);
+		}
+
+		print_task_info(&handle);
+		goto out;
+	}
+
 	process_uftrace_info(&handle, opts, print_info, NULL);
 
 out:
