@@ -104,22 +104,32 @@ int command_live(int argc, char *argv[], struct opts *opts)
 	};
 	int ret;
 
-	umask(022);
-	fd = mkstemp(template);
-	if (fd < 0)
-		pr_err("cannot create temp name");
+	if (!opts->record) {
+		tmp_dirname = template;
+		umask(022);
+		fd = mkstemp(template);
+		if (fd < 0) {
+			if (errno != EPERM)
+				pr_err("cannot access to /tmp");
 
-	close(fd);
-	unlink(template);
+			fd = mkstemp(template + sizeof("/tmp/") - 1);
 
-	tmp_dirname = template;
-	atexit(cleanup_tempdir);
+			if (fd < 0)
+				pr_err("cannot create temp name");
+			tmp_dirname += sizeof("/tmp/") - 1;
+		}
 
-	sa.sa_handler = sigsegv_handler;
-	sigfillset(&sa.sa_mask);
-	sigaction(SIGSEGV, &sa, NULL);
+		close(fd);
+		unlink(tmp_dirname);
 
-	opts->dirname = template;
+		atexit(cleanup_tempdir);
+
+		sa.sa_handler = sigsegv_handler;
+		sigfillset(&sa.sa_mask);
+		sigaction(SIGSEGV, &sa, NULL);
+
+		opts->dirname = tmp_dirname;
+	}
 
 	if (opts->list_event) {
 		if (geteuid() == 0)
