@@ -29,6 +29,9 @@
 #include "utils/kernel.h"
 #include "utils/perf.h"
 
+#ifndef EFD_SEMAPHORE
+# define EFD_SEMAPHORE (1 << 0)
+#endif
 #define SHMEM_NAME_SIZE (64 - (int)sizeof(struct list_head))
 
 struct shmem_list {
@@ -131,12 +134,12 @@ char * get_libmcount_path(struct opts *opts)
 	}
 
 #ifdef INSTALL_LIB_PATH
-	/* try first to load libmcount from the installtion path */
+	/* try first to load libmcount from the installation path */
 	snprintf(lib, PATH_MAX, "%s/%s", INSTALL_LIB_PATH, libmcount);
 	if (access(lib, F_OK) == 0)
 		return lib;
 #endif
-	strcpy(lib, libmcount);
+	strncpy(lib, libmcount, PATH_MAX);
 	return lib;
 }
 
@@ -259,7 +262,7 @@ static void setup_child_environ(struct opts *opts, int argc, char *argv[])
 		setenv("UFTRACE_PLTHOOK", "1", 1);
 
 		if (opts->want_bind_not) {
-			/* do not update GOTPLT after resolving symbols */
+			/* do not update GOT/PLT after resolving symbols */
 			setenv("LD_BIND_NOT", "1", 1);
 		}
 
@@ -1475,8 +1478,8 @@ static void print_child_usage(struct rusage *ru)
 #define UFTRACE_MSG  "Cannot trace '%s': No such executable file.\n"
 
 #define MCOUNT_MSG  "Can't find '%s' symbol in the '%s'.\n"		\
-"\tIt seems not to be compiled with -pg or -finstrument-functions flag\n" 	\
-"\twhich generates traceable code.  Please check your binary file.\n"
+"\tIt seems not to be compiled with -pg or -finstrument-functions flag.\n" 	\
+"\tYou can rebuild your program with it or use -P option for dynamic tracing.\n"
 
 #define UFTRACE_ELF_MSG  "Cannot trace '%s': Invalid file\n"		\
 "\tThis file doesn't look like an executable ELF file.\n"		\
@@ -1726,6 +1729,11 @@ static void setup_writers(struct writer_data *wd, struct opts *opts)
 		wd->sock = -1;
 
 	wd->nr_cpu = sysconf(_SC_NPROCESSORS_ONLN);
+	if (unlikely(wd->nr_cpu <= 0)) {
+		wd->nr_cpu = sysconf(_SC_NPROCESSORS_CONF);
+		if (wd->nr_cpu <= 0)
+			pr_err("cannot know number of cpu");
+	}
 
 	if (opts->kernel || has_kernel_event(opts->event)) {
 		int err;
@@ -1878,12 +1886,12 @@ static int stop_tracing(struct writer_data *wd, struct opts *opts)
 				ret = UFTRACE_EXIT_SUCCESS;
 		}
 		else if (WIFSIGNALED(status)) {
-			pr_yellow("child terminated by signal: %d: %s\n",
+			pr_warn("child terminated by signal: %d: %s\n",
 				  WTERMSIG(status), strsignal(WTERMSIG(status)));
 			ret = UFTRACE_EXIT_SIGNALED;
 		}
 		else {
-			pr_yellow("child terminated with unknown reason: %d\n",
+			pr_warn("child terminated with unknown reason: %d\n",
 				  status);
 			memset(&wd->usage, 0, sizeof(wd->usage));
 			ret = UFTRACE_EXIT_UNKNOWN;
@@ -2158,8 +2166,10 @@ int command_record(int argc, char *argv[], struct opts *opts)
 		else
 			do_child_exec(ready, opts, argc, argv);
 
-		if (channel)
+		if (channel) {
 			unlink(channel);
+			free(channel);
+		}
 		return ret;
 	}
 
@@ -2168,7 +2178,9 @@ int command_record(int argc, char *argv[], struct opts *opts)
 	else
 		ret = do_main_loop(ready, opts, pid);
 
-	if (channel)
+	if (channel) {
 		unlink(channel);
+		free(channel);
+	}
 	return ret;
 }
