@@ -53,7 +53,7 @@ static void print_addr(struct field_data *fd)
 	if (fstack == NULL)  /* LOST */
 		pr_out("%*s", width, "");
 	else
-		pr_out("%*lx", width, fstack->addr);
+		pr_out("%*"PRIx64, width, effective_addr(fstack->addr));
 }
 
 static void print_timestamp(struct field_data *fd)
@@ -253,12 +253,15 @@ static void print_backtrace(struct uftrace_task_reader *task)
 		struct display_field *field;
 		struct sym *sym;
 		char *name;
-		struct fstack *fstack = &task->func_stack[i];
+		struct fstack *fstack = fstack_get(task, i);
 		struct field_data fd = {
 			.task = task,
 			.fstack = fstack,
 		};
 
+		if (fstack == NULL)
+			continue;
+
 		sym = task_find_sym_addr(sessions, task,
 					 fstack->total_time, fstack->addr);
 
@@ -287,7 +290,7 @@ static void print_event(struct uftrace_task_reader *task,
 	char *evt_name = get_event_name(task->h, evt_id);
 
 	if (evt_id == EVENT_ID_EXTERN_DATA) {
-		pr_color(color, "%s: %s", evt_name, task->args.data);
+		pr_color(color, "%s: %s", evt_name, (char *)task->args.data);
 	}
 	else if (evt_id >= EVENT_ID_USER) {
 		/* TODO: some events might have arguments */
@@ -314,66 +317,62 @@ static void print_event(struct uftrace_task_reader *task,
 			u.statm = task->args.data;
 			pr_color(color, "%s (size=%"PRIu64"KB, rss=%"PRIu64"KB, shared=%"PRIu64"KB)",
 				 evt_name, u.statm->vmsize, u.statm->vmrss, u.statm->shared);
-			return;
+			break;
 		case EVENT_ID_READ_PAGE_FAULT:
 			u.page_fault = task->args.data;
 			pr_color(color, "%s (major=%"PRIu64", minor=%"PRIu64")",
 				 evt_name, u.page_fault->major, u.page_fault->minor);
-			return;
+			break;
 		case EVENT_ID_READ_PMU_CYCLE:
 			u.cycle = task->args.data;
 			pr_color(color, "%s (cycle=%"PRIu64", instructions=%"PRIu64")",
 				 evt_name, u.cycle->cycles, u.cycle->instrs);
-			return;
+			break;
 		case EVENT_ID_READ_PMU_CACHE:
 			u.cache = task->args.data;
 			pr_color(color, "%s (refers=%"PRIu64", misses=%"PRIu64")",
 				 evt_name, u.cache->refers, u.cache->misses);
-			return;
+			break;
 		case EVENT_ID_READ_PMU_BRANCH:
 			u.branch = task->args.data;
 			pr_color(color, "%s (branch=%"PRIu64", misses=%"PRIu64")",
 				 evt_name, u.branch->branch, u.branch->misses);
-			return;
+			break;
 		case EVENT_ID_DIFF_PROC_STATM:
 			u.statm = task->args.data;
 			pr_color(color, "%s (size=%+"PRId64"KB, rss=%+"PRId64"KB, shared=%+"PRId64"KB)",
 				 evt_name, u.statm->vmsize, u.statm->vmrss, u.statm->shared);
-			return;
+			break;
 		case EVENT_ID_DIFF_PAGE_FAULT:
 			u.page_fault = task->args.data;
 			pr_color(color, "%s (major=%+"PRId64", minor=%+"PRId64")",
 				 evt_name, u.page_fault->major, u.page_fault->minor);
-			return;
+			break;
 		case EVENT_ID_DIFF_PMU_CYCLE:
 			u.cycle = task->args.data;
 			pr_color(color, "%s (cycle=%+"PRId64", instructions=%+"PRId64", IPC=%.2f)",
 				 evt_name, u.cycle->cycles, u.cycle->instrs,
 				 (float)u.cycle->instrs / u.cycle->cycles);
-			return;
+			break;
 		case EVENT_ID_DIFF_PMU_CACHE:
 			u.cache = task->args.data;
 			pr_color(color, "%s (refers=%+"PRId64", misses=%+"PRId64", hit=%"PRId64"%%)",
 				 evt_name, u.cache->refers, u.cache->misses,
 				 (u.cache->refers - u.cache->misses) * 100 / u.cache->refers);
-			return;
+			break;
 		case EVENT_ID_DIFF_PMU_BRANCH:
 			u.branch = task->args.data;
 			pr_color(color, "%s (branch=%+"PRId64", misses=%+"PRId64", predict=%"PRId64"%%)",
 				 evt_name, u.branch->branch, u.branch->misses,
 				 (u.branch->branch - u.branch->misses) * 100 / u.branch->branch);
-			return;
+			break;
 		case EVENT_ID_WATCH_CPU:
 			u.cpu = task->args.data;
 			pr_color(color, "%s (cpu=%d)", evt_name, *u.cpu);
-			return;
-
+			break;
 		default:
 			pr_color(color, "%s", evt_name);
-			break;
 		}
-		pr_color(color, "user_event:%u", evt_id);
-		return;
 	}
 	else {
 		/* kernel events */
@@ -396,7 +395,10 @@ static int print_flat_rstack(struct uftrace_data *handle,
 
 	sym = task_find_sym(sessions, task, rstack);
 	name = symbol_getname(sym, rstack->addr);
-	fstack = &task->func_stack[rstack->depth];
+	fstack = fstack_get(task, rstack->depth);
+
+	if (fstack == NULL)
+		goto out;
 
 	/* skip it if --no-libcall is given */
 	if (!opts->libcall && sym && sym->type == ST_PLT_FUNC)
@@ -442,10 +444,13 @@ static void print_task_newline(int current_tid)
 	prev_tid = current_tid;
 }
 
+#define print_char(c)                                                   \
+({ args[n] = c; n++; len--; })
+
 #define print_args(fmt, ...)						\
 ({ int _x = snprintf(args + n, len, fmt, ##__VA_ARGS__); n += _x; len -= _x; })
 
-#define print_escaped_char(c)						\
+#define print_json_escaped_char(c)					\
 	do {								\
 		if (c == '\n')						\
 			print_args("\\\\n");				\
@@ -456,11 +461,23 @@ static void print_task_newline(int current_tid)
 		else if (c == '"')					\
 			print_args("\\\"");				\
 		else if (isprint(c))					\
-			print_args("%c", c);				\
+			print_char(c);                                  \
 		else							\
 			print_args("\\\\x%02hhx", c);			\
 	} while (0)
 
+#define print_escaped_char(c)						\
+	do {								\
+		if (c == '\0')						\
+			print_args("\\0");				\
+		else if (c == '\b')					\
+			print_args("\\b");				\
+		else if (c == '\n')                                     \
+			print_args("\\n");				\
+		else							\
+			print_char(c);                                  \
+	} while (0)
+
 void get_argspec_string(struct uftrace_task_reader *task,
 			char *args, size_t len,
 			enum argspec_string_bits str_mode)
@@ -487,7 +504,7 @@ void get_argspec_string(struct uftrace_task_reader *task,
 	bool has_more         = !!(str_mode & HAS_MORE);
 	bool is_retval        = !!(str_mode & IS_RETVAL);
 	bool needs_assignment = !!(str_mode & NEEDS_ASSIGNMENT);
-	bool needs_escape     = !!(str_mode & NEEDS_ESCAPE);
+	bool needs_json       = !!(str_mode & NEEDS_JSON);
 
 	if (!has_more) {
 		if (needs_paren)
@@ -565,39 +582,51 @@ void get_argspec_string(struct uftrace_task_reader *task,
 		if (spec->fmt == ARG_FMT_STR ||
 		    spec->fmt == ARG_FMT_STD_STRING) {
 			unsigned short slen;
-			unsigned short newline = 0;
-			char last_ch;
 
 			memcpy(&slen, data, 2);
 
-			last_ch = *((char *)data + slen + 1);
-			if (last_ch == '\n')
-				newline = 1;
-
-			str = xmalloc(slen + newline + 1);
+			str = xmalloc(slen + 1);
 			memcpy(str, data + 2, slen);
 			str[slen] = '\0';
 
-			if (newline) {
-				str[slen - 1] = '\\';
-				str[slen]     = 'n';
-				str[slen + 1] = '\0';
-			}
-
-			if (!memcmp(str, &null_str, sizeof(null_str)))
+			if (slen == 4 && !memcmp(str, &null_str, sizeof(null_str)))
 				print_args("NULL");
-			else if (needs_escape) {
+			else if (needs_json) {
 				char *p = str;
 				print_args("\\\"");
 				while (*p) {
 					char c = *p++;
-					print_escaped_char(c);
+					print_json_escaped_char(c);
 				}
 				print_args("\\\"");
 			}
 			else {
 				print_args("%s", color_string);
-				print_args("\"%.*s\"", slen + newline, str);
+				print_args("\"");
+
+				char *p = str;
+				while (*p) {
+					char c = *p++;
+					if (c & 0x80) {
+						break;
+					}
+				}
+				/*
+				* if value of character is over than 128(0x80),
+				* then it will be UTF-8 string
+				*/
+				if (*p) {
+					print_args("%.*s", slen, str);
+				}
+				else {
+					p = str;
+					while (*p) {
+						char c = *p++;
+						print_escaped_char(c);
+					}
+				}
+
+				print_args("\"");
 				print_args("%s", color_reset);
 			}
 
@@ -612,14 +641,16 @@ void get_argspec_string(struct uftrace_task_reader *task,
 			char c;
 
 			memcpy(&c, data, 1);
-			if (needs_escape) {
+			if (needs_json) {
 				print_args("'");
-				print_escaped_char(c);
+				print_json_escaped_char(c);
 				print_args("'");
 			}
 			else {
 				print_args("%s", color_string);
-				print_args("'%c'", c);
+				print_args("'");
+				print_escaped_char(c);
+				print_args("'");
 				print_args("%s", color_reset);
 			}
 			size = 1;
@@ -820,7 +851,7 @@ static int print_graph_rstack(struct uftrace_data *handle,
 			str_mode |= HAS_MORE;
 		get_argspec_string(task, args, sizeof(args), str_mode);
 
-		fstack = &task->func_stack[task->stack_count - 1];
+		fstack = fstack_get(task, task->stack_count - 1);
 
 		if (!opts->no_merge)
 			next = fstack_skip(handle, task, rstack_depth, opts);
@@ -880,9 +911,10 @@ static int print_graph_rstack(struct uftrace_data *handle,
 		struct fstack *fstack;
 
 		/* function exit */
-		fstack = &task->func_stack[task->stack_count];
+		fstack = fstack_get(task, task->stack_count);
 
-		if (!(fstack->flags & FSTACK_FL_NORECORD) && fstack_enabled) {
+		if (fstack_enabled && fstack != NULL &&
+		    !(fstack->flags & FSTACK_FL_NORECORD)) {
 			int depth = fstack_update(UFTRACE_EXIT, task, fstack);
 			char *retval = args;
 
@@ -974,9 +1006,10 @@ lost:
 			depth = 0;
 
 		/* for sched-in to show schedule duration */
-		fstack = &task->func_stack[task->stack_count];
+		fstack = fstack_get(task, task->stack_count);
 
-		if (!(fstack->flags & FSTACK_FL_NORECORD) && fstack_enabled) {
+		if (fstack_enabled && fstack != NULL &&
+		    !(fstack->flags & FSTACK_FL_NORECORD)) {
 			if (evt_id == EVENT_ID_PERF_SCHED_IN &&
 			    fstack->total_time)
 				print_field(task, fstack, NULL);
@@ -1004,18 +1037,18 @@ static void print_warning(struct uftrace_task_reader *task)
 
 static bool skip_sys_exit(struct opts *opts, struct uftrace_task_reader *task)
 {
-	uint64_t ip;
 	struct sym *sym;
+	struct fstack *fstack;
 
-	if (task->func_stack == NULL)
+	fstack = fstack_get(task, 0);
+	if (fstack == NULL)
 		return true;
 
 	/* skip 'sys_exit[_group] at last for kernel tracing */
 	if (!has_kernel_data(task->h->kernel) || task->user_stack_count != 0)
 		return false;
 
-	ip = task->func_stack[0].addr;
-	sym = find_symtabs(&task->h->sessions.first->symtabs, ip);
+	sym = find_symtabs(&task->h->sessions.first->symtabs, fstack->addr);
 	if (sym == NULL)
 		return false;
 
@@ -1043,7 +1076,10 @@ static void print_remaining_stack(struct opts *opts,
 			continue;
 
 		for (k = 0; k < task->stack_count; k++) {
-			if (task->func_stack[k].addr)
+			struct fstack *fstack;
+
+			fstack = fstack_get(task, k);
+			if (fstack != NULL && fstack->addr != 0)
 				break;
 			zero_count++;
 		}
@@ -1059,13 +1095,15 @@ static void print_remaining_stack(struct opts *opts,
 
 	for (i = 0; i < handle->nr_tasks; i++) {
 		struct uftrace_task_reader *task = &handle->tasks[i];
+		struct fstack *fstack;
 		int zero_count = 0;
 
 		if (task->stack_count == 0)
 			continue;
 
 		for (k = 0; k < task->stack_count; k++) {
-			if (task->func_stack[k].addr)
+			fstack = fstack_get(task, k);
+			if (fstack != NULL && fstack->addr != 0)
 				break;
 			zero_count++;
 		}
@@ -1079,12 +1117,17 @@ static void print_remaining_stack(struct opts *opts,
 		pr_out("task: %d\n", task->tid);
 
 		while (task->stack_count-- > 0) {
-			struct fstack *fstack = &task->func_stack[task->stack_count];
-			uint64_t time = fstack->total_time;
-			uint64_t ip = fstack->addr;
+			uint64_t time;
+			uint64_t ip;
 			struct sym *sym;
 			char *symname;
 
+			fstack = fstack_get(task, task->stack_count);
+			if (fstack == NULL)
+				continue;
+
+			time = fstack->total_time;
+			ip = fstack->addr;
 			sym = task_find_sym_addr(sessions, task, time, ip);
 			symname = symbol_getname(sym, ip);
 
@@ -1120,7 +1163,7 @@ int command_replay(int argc, char *argv[], struct opts *opts)
 		    field_table, ARRAY_SIZE(field_table));
 
 	if (!opts->flat && peek_rstack(&handle, &task) == 0)
-		print_header(&output_fields, "#", 1);
+		print_header(&output_fields, "#", "FUNCTION", 1);
 
 	while (read_rstack(&handle, &task) == 0 && !uftrace_done) {
 		struct uftrace_record *rstack = task->rstack;
