@@ -55,7 +55,7 @@ static int run_script_for_rstack(struct uftrace_data *handle,
 		/* display depth is set in fstack_entry() */
 		depth = task->display_depth;
 
-		fstack = &task->func_stack[task->stack_count - 1];
+		fstack = fstack_get(task ,task->stack_count - 1);
 		fstack_update(UFTRACE_ENTRY, task, fstack);
 
 		if (!script_match_filter(symname))
@@ -83,9 +83,10 @@ static int run_script_for_rstack(struct uftrace_data *handle,
 		struct fstack *fstack;
 
 		/* function exit */
-		fstack = &task->func_stack[task->stack_count];
+		fstack = fstack_get(task, task->stack_count);
 
-		if (!(fstack->flags & FSTACK_FL_NORECORD) && fstack_enabled) {
+		if (fstack_enabled && fstack &&
+		    !(fstack->flags & FSTACK_FL_NORECORD)) {
 			int depth = fstack_update(UFTRACE_EXIT, task, fstack);
 
 			if (!script_match_filter(symname)) {
@@ -145,7 +146,7 @@ int command_script(int argc, char *argv[], struct opts *opts)
 	}
 
 	if (!opts->script_file) {
-		pr_out("Usage: uftrace script [-S|--script] [<script_file>]\n");
+		pr_out("Usage: uftrace script (-S|--script) <script_file>\n");
 		return -1;
 	}
 
@@ -180,8 +181,10 @@ int command_script(int argc, char *argv[], struct opts *opts)
 	strv_copy(&info.cmds, argc, argv);
 
 	/* initialize script */
-	if (script_init(&info, opts->patt_type) < 0)
-		return -1;
+	if (script_init(&info, opts->patt_type) < 0) {
+		ret = -1;
+		goto out;
+	}
 
 	while (read_rstack(&handle, &task) == 0 && !uftrace_done) {
 		if (!fstack_check_opts(task, opts))
@@ -195,10 +198,12 @@ int command_script(int argc, char *argv[], struct opts *opts)
 
 	/* dtor for script support */
 	script_uftrace_end();
-
+out:
 	script_finish();
 
 	close_data_file(opts, &handle);
 
+	strv_free(&info.cmds);
+
 	return ret;
 }
