@@ -142,6 +142,7 @@ static const char *help[] = {
 	"g             Show call graph for this function",
 	"R             Show uftrace report",
 	"r             Show uftrace report for this function",
+	"s             Sort by the next column in report",
 	"I             Show uftrace info",
 	"S             Change session",
 	"O             Open editor",
@@ -159,6 +160,9 @@ static const char *help[] = {
 	"q             Quit",
 };
 
+static const char *report_sort_key[] = { OPT_SORT_KEYS, "self", "call" };
+static int curr_sort_key = 0;
+
 static void init_colors(void)
 {
 	if (!has_colors())
@@ -231,7 +235,7 @@ static void print_graph_addr(struct field_data *fd)
 	/* uftrace records (truncated) 48-bit addresses */
 	int width = sizeof(long) == 4 ? 8 : 12;
 
-	printw("%*lx", width, node->addr);
+	printw("%*"PRIx64, width, effective_addr(node->addr));
 }
 
 static struct display_field field_total_time= {
@@ -477,7 +481,9 @@ static void add_remaining_node(struct opts *opts, struct uftrace_data *handle)
 			last_time = handle->time_range.stop;
 
 		while (--task->stack_count >= 0) {
-			fstack = &task->func_stack[task->stack_count];
+			fstack = fstack_get(task, task->stack_count);
+			if (fstack == NULL)
+				continue;
 
 			if (fstack->addr == 0)
 				continue;
@@ -693,7 +699,7 @@ static void build_partial_graph(struct tui_report_node *root_node,
 			parent = (void *)parent->n.parent;
 		}
 
-		/* but, unfoled it if it's the last child */
+		/* but, unfolded it if it's the last child */
 		if (n == 2)
 			tmp->folded = false;
 	}
@@ -1082,7 +1088,7 @@ static void win_footer_graph(struct tui_window *win,
 	graph->disp_update = false;
 }
 
-static void print_graph_field(struct uftrace_graph_node *node)
+static void print_graph_field(struct uftrace_graph_node *node, int width)
 {
 	struct display_field *field;
 	struct field_data fd = {
@@ -1093,18 +1099,29 @@ static void print_graph_field(struct uftrace_graph_node *node)
 		return;
 
 	list_for_each_entry(field, &graph_output_fields, list) {
-		printw("%*s", FIELD_SPACE, "");
-		field->print(&fd);
+		if (width >= FIELD_SPACE) {
+			printw("%*s", FIELD_SPACE, "");
+			width -= FIELD_SPACE;
+		}
+
+		if (width >= field->length) {
+			field->print(&fd);
+			width -= field->length;
+		}
 	}
-	printw(FIELD_SEP);
+
+	if (width >= FIELD_SPACE)
+		printw(FIELD_SEP);
 }
 
-static void print_graph_empty(void)
+static void print_graph_empty(struct tui_graph *graph, int width)
 {
 	struct display_field *field;
 
 	if (list_empty(&graph_output_fields))
 		return;
+	if (graph->width > width)
+		return;
 
 	list_for_each_entry(field, &graph_output_fields, list)
 		printw("%*s", field->length + FIELD_SPACE, "");
@@ -1114,12 +1131,18 @@ static void print_graph_empty(void)
 
 static void print_graph_indent(struct tui_graph *graph,
 			       struct tui_graph_node *node,
-			       int depth, bool single_child)
+			       int width, int depth, bool single_child)
 {
 	int i;
 	struct tui_graph_node *parent = (void *)node->n.parent;
 
 	for (i = 0; i < depth; i++) {
+		if (width < 3) {
+			printw("%*.*s", width, width, "   ");
+			break;
+		}
+		width -= 3;
+
 		if (!graph->disp_mask[i]) {
 			printw("   ");
 			continue;
@@ -1146,8 +1169,8 @@ static void win_display_graph(struct tui_window *win, void *node)
 	int width;
 
 	if (node == NULL) {
-		print_graph_empty();
-		print_graph_indent(graph, graph->disp, d, true);
+		print_graph_empty(graph, COLS);
+		print_graph_indent(graph, graph->disp, COLS - w, d, true);
 		return;
 	}
 
@@ -1162,8 +1185,8 @@ static void win_display_graph(struct tui_window *win, void *node)
 			fold_sign = " ";
 	}
 
-	print_graph_field(&curr->n);
-	print_graph_indent(graph, curr, d, single_child);
+	print_graph_field(&curr->n, COLS);
+	print_graph_indent(graph, curr, COLS - w, d, single_child);
 
 	width = d * 3 + w;
 
@@ -1175,14 +1198,22 @@ static void win_display_graph(struct tui_window *win, void *node)
 	else {
 		char buf[32];
 
-		/* 4 = fold_sign(1) + parenthesis92) + space(1) */
-		width += snprintf(buf, sizeof(buf),
-				  "%d", curr->n.nr_calls) + 4;
-		width = COLS - width;
+		if (width < COLS) {
+			w = COLS - width;
+			width += snprintf(buf, sizeof(buf), "%s(%d) ",
+					  fold_sign, curr->n.nr_calls);
 
-		if (width > 0) {
-			printw("%s(%d) %-*.*s", fold_sign, curr->n.nr_calls,
-			       width, width, curr->n.name);
+			/* handle UTF-8 character length */
+			if (strcmp(fold_sign, " ")) {
+				width -= 2;
+				w += 2;
+			}
+			printw("%.*s", w, buf);
+		}
+
+		if (width < COLS) {
+			w = COLS - width;
+			printw("%-*.*s", w, w, curr->n.name);
 		}
 	}
 }
@@ -1277,7 +1308,7 @@ static struct tui_report * tui_report_init(struct opts *opts)
 {
 	struct tui_window *win = &tui_report.win;
 
-	report_setup_sort("total");
+	report_setup_sort(OPT_SORT_KEYS);
 	report_sort_nodes(&tui_report.name_tree, &tui_report.sort_tree);
 
 	tui_window_init(win, &report_ops);
@@ -1329,8 +1360,10 @@ static bool win_search_report(struct tui_window *win, void *node, char *str)
 static void win_header_report(struct tui_window *win, struct uftrace_data *handle)
 {
 	int w = 46;
+	char header[][32] = { " Total Time", " Self Time", " Calls" };
 
-	printw("  %10s  %10s  %10s  %s", "Total Time", "Self Time", "Calls", "Function");
+	header[curr_sort_key][0] = '*';
+	printw(" %11s %11s %11s  %s", header[0], header[1], header[2], "Function");
 	if (COLS > w)
 		printw("%*s", COLS - w, "");
 }
@@ -2334,7 +2367,9 @@ static void tui_main_loop(struct opts *opts, struct uftrace_data *handle)
 	session = tui_session_init(opts);
 
 	/* start with graph only if there's one session */
-	if (session->nr_node > 1)
+	if (opts->report)
+		win = &report->win;
+	else if (session->nr_node > 1)
 		win = &session->win;
 	else
 		win = &graph->win;
@@ -2463,6 +2498,18 @@ static void tui_main_loop(struct opts *opts, struct uftrace_data *handle)
 				full_redraw = true;
 			}
 			break;
+		case 's':
+			if (!tui_window_change(win, &report->win)) {
+				curr_sort_key = (curr_sort_key + 1) %
+						ARRAY_SIZE(report_sort_key);
+				report_setup_sort(report_sort_key[curr_sort_key]);
+				report_sort_nodes(&tui_report.name_tree,
+						  &tui_report.sort_tree);
+
+				tui_window_move_home(win);
+				full_redraw = true;
+			}
+			break;
 		case 'I':
 			if (tui_window_change(win, &info->win)) {
 				win = &info->win;
