@@ -21,6 +21,7 @@ static struct mcount_dynamic_stats {
 	int failed;
 	int skipped;
 	int nomatch;
+	int unpatch;
 } stats;
 
 #define PAGE_SIZE   4096
@@ -66,12 +67,22 @@ static struct mcount_orig_insn *lookup_code(struct rb_root *root,
 	return iter;
 }
 
-struct mcount_orig_insn *mcount_save_code(unsigned long addr, unsigned insn_size,
+struct mcount_orig_insn *mcount_save_code(struct mcount_disasm_info *info,
 					  void *jmp_insn, unsigned jmp_size)
 {
 	struct code_page *cp = NULL;
 	struct mcount_orig_insn *orig;
-	const int patch_size = ALIGN(insn_size + jmp_size, 32);
+	int patch_size;
+
+	if (unlikely(info->modified)) {
+		/* it needs to save original instructions as well */
+		int orig_size = ALIGN(info->orig_size, 16);
+		int copy_size = ALIGN(info->copy_size + jmp_size, 16);
+		patch_size = ALIGN(copy_size + orig_size, 32);
+	}
+	else {
+		patch_size = ALIGN(info->copy_size + jmp_size, 32);
+	}
 
 	if (!list_empty(&code_pages))
 		cp = list_last_entry(&code_pages, struct code_page, list);
@@ -87,11 +98,20 @@ struct mcount_orig_insn *mcount_save_code(unsigned long addr, unsigned insn_size
 		list_add_tail(&cp->list, &code_pages);
 	}
 
-	orig = lookup_code(&code_tree, addr, true);
+	orig = lookup_code(&code_tree, info->addr, true);
 	orig->insn = cp->page + cp->pos;
+	orig->orig = orig->insn;
+	orig->orig_size = info->orig_size;
+	orig->insn_size = info->copy_size + jmp_size;
 
-	memcpy(orig->insn, (void *)addr, insn_size);
-	memcpy(orig->insn + insn_size, jmp_insn, jmp_size);
+	if (info->modified) {
+		/* save original instructions before modification */
+		orig->orig = orig->insn + patch_size - ALIGN(info->orig_size, 16);
+		memcpy(orig->orig, (void *)info->addr, info->orig_size);
+	}
+
+	memcpy(orig->insn, info->insns, info->copy_size);
+	memcpy(orig->insn + info->copy_size, jmp_insn, jmp_size);
 
 	cp->pos += patch_size;
 	return orig;
@@ -102,7 +122,7 @@ void mcount_freeze_code(void)
 	struct code_page *cp;
 
 	list_for_each_entry(cp, &code_pages, list)
-		mprotect(cp->page, CODE_CHUNK, PROT_EXEC);
+		mprotect(cp->page, CODE_CHUNK, PROT_READ|PROT_EXEC);
 }
 
 void *mcount_find_code(unsigned long addr)
@@ -116,6 +136,11 @@ void *mcount_find_code(unsigned long addr)
 	return orig->insn;
 }
 
+struct mcount_orig_insn * mcount_find_insn(unsigned long addr)
+{
+	return lookup_code(&code_tree, addr, false);
+}
+
 /* dummy functions (will be overridden by arch-specific code) */
 __weak int mcount_setup_trampoline(struct mcount_dynamic_info *mdi)
 {
@@ -133,6 +158,12 @@ __weak int mcount_patch_func(struct mcount_dynamic_info *mdi, struct sym *sym,
 	return -1;
 }
 
+__weak int mcount_unpatch_func(struct mcount_dynamic_info *mdi, struct sym *sym,
+			       struct mcount_disasm_engine *disasm)
+{
+	return -1;
+}
+
 __weak void mcount_arch_find_module(struct mcount_dynamic_info *mdi,
 				    struct symtab *symtab)
 {
@@ -188,6 +219,7 @@ static int find_dynamic_module(struct dl_phdr_info *info, size_t sz, void *data)
 	}
 	mdi->base_addr += info->dlpi_addr;
 	mdi->text_addr += info->dlpi_addr;
+	INIT_LIST_HEAD(&mdi->bad_syms);
 
 	map = find_map(symtabs, mdi->base_addr);
 	if (map && map->mod) {
@@ -234,13 +266,42 @@ struct mcount_dynamic_info *setup_trampoline(struct uftrace_mmap *map)
 	return mdi;
 }
 
+struct patt_list {
+	struct list_head list;
+	struct uftrace_pattern patt;
+	char *module;
+	bool positive;
+};
+
+static bool match_pattern_list(struct list_head *patterns,
+			       struct uftrace_mmap *map,
+			       char *sym_name)
+{
+	struct patt_list *pl;
+	bool ret = false;
+
+	list_for_each_entry(pl, patterns, list) {
+		char *libname = basename(map->libname);
+
+		if (strncmp(libname, pl->module, strlen(pl->module)))
+			continue;
+
+		if (match_filter_pattern(&pl->patt, sym_name))
+			ret = pl->positive;
+	}
+
+	return ret;
+}
+
 static int do_dynamic_update(struct symtabs *symtabs, char *patch_funcs,
 			     enum uftrace_pattern_type ptype,
 			     struct mcount_disasm_engine *disasm,
 			     unsigned min_size)
 {
+	struct uftrace_mmap *map;
 	struct symtab *symtab;
 	struct strv funcs = STRV_INIT;
+	char *def_mod;
 	char *name;
 	int j;
 	/* skip special startup (csu) functions */
@@ -249,47 +310,68 @@ static int do_dynamic_update(struct symtabs *symtabs, char *patch_funcs,
 		"__libc_csu_init",
 		"__libc_csu_fini",
 	};
+	LIST_HEAD(patterns);
+	struct patt_list *pl;
+	bool all_negative = true;
 
 	if (patch_funcs == NULL)
 		return 0;
 
+	def_mod = basename(symtabs->exec_map->libname);
 	strv_split(&funcs, patch_funcs, ";");
 
 	strv_for_each(&funcs, name, j) {
-		bool found = false;
-		bool csu_skip;
-		char *modname, *delim;
-		unsigned i, k;
-		struct sym *sym;
-		struct uftrace_pattern patt;
-		struct uftrace_mmap *map;
-		struct mcount_dynamic_info *mdi = NULL;
+		char *delim;
+
+		pl = xzalloc(sizeof(*pl));
+
+		if (name[0] == '!')
+			name++;
+		else {
+			pl->positive = true;
+			all_negative = false;
+		}
 
 		delim = strchr(name, '@');
-
 		if (delim == NULL) {
-			/* first of uftrace_mmap is main module always. */
-			map = symtabs->maps;
-			symtab = &symtabs->maps->mod->symtab;
+			pl->module = xstrdup(def_mod);
 		}
 		else {
 			*delim = '\0';
-			modname = ++delim;
-			map = find_map_by_name(symtabs, modname);
-			if (map == NULL || map->mod == NULL) {
-				pr_dbg("Failed to find map of %s\n", modname);
-				continue;
-			}
-			symtab = &map->mod->symtab;
+			pl->module = xstrdup(++delim);
 		}
 
+		init_filter_pattern(ptype, &pl->patt, name);
+		list_add_tail(&pl->list, &patterns);
+	}
+
+	/* prepend match-all pattern, if all patterns are negative */
+	if (all_negative) {
+		pl = xzalloc(sizeof(*pl));
+		pl->positive = true;
+		pl->module = xstrdup(def_mod);
+
+		if (ptype == PATT_REGEX)
+			init_filter_pattern(ptype, &pl->patt, ".");
+		else
+			init_filter_pattern(PATT_GLOB, &pl->patt, "*");
+
+		list_add(&pl->list, &patterns);
+	}
+
+	for_each_map(symtabs, map) {
+		bool found = false;
+		bool csu_skip;
+		unsigned i, k;
+		struct sym *sym;
+		struct mcount_dynamic_info *mdi;
+
+		/* TODO: filter out unsuppported libs */
 		mdi = setup_trampoline(map);
-		if (mdi == NULL) {
-			pr_warn("Failed to set the trampoline into %s\n", map->libname);
+		if (mdi == NULL)
 			continue;
-		}
 
-		init_filter_pattern(ptype, &patt, name);
+		symtab = &map->mod->symtab;
 
 		for (i = 0; i < symtab->nr_sym; i++) {
 			sym = &symtab->sym[i];
@@ -308,8 +390,11 @@ static int do_dynamic_update(struct symtabs *symtabs, char *patch_funcs,
 			    sym->type != ST_GLOBAL_FUNC)
 				continue;
 
-			if (!match_filter_pattern(&patt, sym->name))
+			if (!match_pattern_list(&patterns, map, sym->name)) {
+				if (mcount_unpatch_func(mdi, sym, disasm) == 0)
+					stats.unpatch++;
 				continue;
+			}
 
 			found = true;
 			switch (mcount_patch_func(mdi, sym, disasm, min_size)) {
@@ -328,8 +413,6 @@ static int do_dynamic_update(struct symtabs *symtabs, char *patch_funcs,
 
 		if (!found)
 			stats.nomatch++;
-
-		free_filter_pattern(&patt);
 	}
 
 	if (stats.failed + stats.skipped + stats.nomatch == 0) {
@@ -337,6 +420,16 @@ static int do_dynamic_update(struct symtabs *symtabs, char *patch_funcs,
 		       stats.total, basename(symtabs->filename));
 	}
 
+	while (!list_empty(&patterns)) {
+		struct patt_list *pl;
+
+		pl = list_first_entry(&patterns, struct patt_list, list);
+
+		list_del(&pl->list);
+		free(pl->module);
+		free(pl);
+	}
+
 	strv_free(&funcs);
 	return 0;
 }
@@ -405,3 +498,49 @@ int mcount_dynamic_update(struct symtabs *symtabs, char *patch_funcs,
 	finish_dynamic_update(disasm);
 	return ret;
 }
+
+struct dynamic_bad_symbol * mcount_find_badsym(struct mcount_dynamic_info *mdi,
+					       unsigned long addr)
+{
+	struct sym *sym;
+	struct dynamic_bad_symbol *badsym;
+
+	sym = find_sym(&mdi->map->mod->symtab, addr - mdi->map->start);
+	if (sym == NULL)
+		return NULL;
+
+	list_for_each_entry(badsym, &mdi->bad_syms, list) {
+		if (badsym->sym == sym)
+			return badsym;
+	}
+
+	return NULL;
+}
+
+bool mcount_add_badsym(struct mcount_dynamic_info *mdi, unsigned long callsite,
+		       unsigned long target)
+{
+	struct sym *sym;
+	struct dynamic_bad_symbol *badsym;
+
+	if (mcount_find_badsym(mdi, target))
+		return true;
+
+	sym = find_sym(&mdi->map->mod->symtab, target - mdi->map->start);
+	if (sym == NULL)
+		return true;
+
+	/* only care about jumps to the middle of a function */
+	if (sym->addr + mdi->map->start == target)
+		return false;
+
+	pr_dbg2("bad jump: %s:%lx to %lx\n", sym ? sym->name : "<unknown>",
+		callsite - mdi->map->start, target - mdi->map->start);
+
+	badsym = xmalloc(sizeof(*badsym));
+	badsym->sym = sym;
+	badsym->reverted = false;
+
+	list_add_tail(&badsym->list, &mdi->bad_syms);
+	return true;
+}
