@@ -126,6 +126,7 @@ struct mcount_thread_data {
 	bool				recursion_marker;
 	bool				in_exception;
 	bool				dead;
+	bool				warned;
 	unsigned long			cygprof_dummy;
 	struct mcount_ret_stack		*rstack;
 	void				*argbuf;
@@ -137,6 +138,7 @@ struct mcount_thread_data {
 	struct mcount_mem_regions	mem_regions;
 	struct mcount_watchpoint	watch;
 	struct mcount_arch_context	arch;
+	struct list_head		pmu_fds;
 };
 
 #ifdef HAVE_MCOUNT_ARCH_CONTEXT
@@ -386,6 +388,9 @@ void save_trigger_read(struct mcount_thread_data *mtdp,
 		       enum trigger_read_type type, bool diff);
 #endif  /* DISABLE_MCOUNT_FILTER */
 
+bool check_mem_region(struct mcount_arg_context *ctx,
+		      unsigned long addr);
+
 void save_watchpoint(struct mcount_thread_data *mtdp,
 		     struct mcount_ret_stack *rstack,
 		     unsigned long watchpoints);
@@ -397,6 +402,7 @@ struct mcount_dynamic_info {
 	unsigned long text_addr;
 	int text_size;
 	unsigned long trampoline;
+	struct list_head bad_syms;
 	void *arch;
 };
 
@@ -417,12 +423,36 @@ int mcount_dynamic_update(struct symtabs *symtabs, char *patch_funcs,
 struct mcount_orig_insn {
 	struct rb_node		node;
 	unsigned long		addr;
+	void			*orig;
 	void			*insn;
+	int			orig_size;
+	int			insn_size;
 };
 
-struct mcount_orig_insn *mcount_save_code(unsigned long addr, unsigned insn_size,
+/*
+ * mcount_disasm_info - information for dynamic patch
+ * @sym : symbol for the function
+ * @addr : currently targeted function address.
+ * @insns : byte array to store instruction.
+ * @orig_size : size of original instructions
+ * @copy_size : size of copied instructions (may be modified)
+ * @modified : whether instruction is changed
+ * @has_jump : whether jump_target should be added
+ */
+struct mcount_disasm_info {
+	struct sym		*sym;
+	unsigned long		addr;
+	unsigned char   	insns[64];
+	int			orig_size;
+	int			copy_size;
+	bool			modified;
+	bool			has_jump;
+};
+
+struct mcount_orig_insn *mcount_save_code(struct mcount_disasm_info *info,
 					  void *jmp_insn, unsigned jmp_size);
 void *mcount_find_code(unsigned long addr);
+struct mcount_orig_insn * mcount_find_insn(unsigned long addr);
 void mcount_freeze_code(void);
 
 /* these should be implemented for each architecture */
@@ -435,6 +465,17 @@ int mcount_patch_func(struct mcount_dynamic_info *mdi, struct sym *sym,
 void mcount_disasm_init(struct mcount_disasm_engine *disasm);
 void mcount_disasm_finish(struct mcount_disasm_engine *disasm);
 
+struct dynamic_bad_symbol {
+	struct list_head	list;
+	struct sym		*sym;
+	bool			reverted;
+};
+
+struct dynamic_bad_symbol * mcount_find_badsym(struct mcount_dynamic_info *mdi,
+					       unsigned long addr);
+bool mcount_add_badsym(struct mcount_dynamic_info *mdi, unsigned long callsite,
+		       unsigned long target);
+
 struct mcount_event_info {
 	char *module;
 	char *provider;
@@ -457,8 +498,9 @@ int mcount_arch_enable_event(struct mcount_event_info *mei);
 
 void mcount_hook_functions(void);
 
-int prepare_pmu_event(enum uftrace_event_id id);
-int read_pmu_event(enum uftrace_event_id id, void *buf);
-void finish_pmu_event(void);
+int read_pmu_event(struct mcount_thread_data *mtdp, enum uftrace_event_id id,
+		   void *buf);
+void release_pmu_event(struct mcount_thread_data *mtdp, enum uftrace_event_id id);
+void finish_pmu_event(struct mcount_thread_data *mtdp);
 
 #endif /* UFTRACE_MCOUNT_INTERNAL_H */
