@@ -102,7 +102,8 @@ static void mcount_filter_init(enum uftrace_pattern_type ptype, char *dirname,
 	if (getenv("UFTRACE_SRCLINE") == NULL)
 		return;
 
-	symtabs.maps->mod = load_module_symtab(&symtabs, symtabs.maps->libname);
+	symtabs.exec_map->mod = load_module_symtab(&symtabs,
+						   symtabs.exec_map->libname);
 
 	/* use debug info if available */
 	prepare_debug_info(&symtabs, ptype, NULL, NULL, false, force);
@@ -116,30 +117,6 @@ static void mcount_filter_finish(void)
 
 #else
 
-static void prepare_pmu_trigger(struct rb_root *root)
-{
-	struct rb_node *node = rb_first(root);
-	struct uftrace_filter *entry;
-
-	while (node) {
-		entry = rb_entry(node, typeof(*entry), node);
-
-		if (entry->trigger.flags & TRIGGER_FL_READ) {
-			if (entry->trigger.read & TRIGGER_READ_PMU_CYCLE)
-				if (prepare_pmu_event(EVENT_ID_READ_PMU_CYCLE) < 0)
-					break;
-			if (entry->trigger.read & TRIGGER_READ_PMU_CACHE)
-				if (prepare_pmu_event(EVENT_ID_READ_PMU_CACHE) < 0)
-					break;
-			if (entry->trigger.read & TRIGGER_READ_PMU_BRANCH)
-				if (prepare_pmu_event(EVENT_ID_READ_PMU_BRANCH) < 0)
-					break;
-		}
-
-		node = rb_next(node);
-	}
-}
-
 /* be careful: this can be called from signal handler */
 static void mcount_finish_trigger(void)
 {
@@ -473,8 +450,6 @@ static void mcount_filter_init(enum uftrace_pattern_type ptype, char *dirname,
 
 	if (getenv("UFTRACE_DISABLED"))
 		mcount_enabled = false;
-
-	prepare_pmu_trigger(&mcount_triggers);
 }
 
 static void mcount_filter_setup(struct mcount_thread_data *mtdp)
@@ -483,12 +458,14 @@ static void mcount_filter_setup(struct mcount_thread_data *mtdp)
 	mtdp->filter.time   = mcount_threshold;
 	mtdp->enable_cached = mcount_enabled;
 	mtdp->argbuf        = xmalloc(mcount_rstack_max * ARGBUF_SIZE);
+	INIT_LIST_HEAD(&mtdp->pmu_fds);
 }
 
 static void mcount_filter_release(struct mcount_thread_data *mtdp)
 {
 	free(mtdp->argbuf);
 	mtdp->argbuf = NULL;
+	finish_pmu_event(mtdp);
 }
 
 static void mcount_filter_finish(void)
@@ -498,7 +475,6 @@ static void mcount_filter_finish(void)
 
 	finish_debug_info(&symtabs);
 
-	finish_pmu_event();
 	mcount_signal_finish();
 }
 
@@ -683,6 +659,7 @@ static void segv_handler(int sig, siginfo_t *si, void *ctx)
 	struct mcount_thread_data *mtdp;
 	struct mcount_ret_stack *rstack;
 	int idx;
+	int i;
 
 	/* set line buffer mode not to discard crash message */
 	setlinebuf(outfp);
@@ -701,44 +678,41 @@ static void segv_handler(int sig, siginfo_t *si, void *ctx)
 	rstack = &mtdp->rstack[idx];
 	record_trace_data(mtdp, rstack, NULL);
 
-	if (dbg_domain[PR_DOMAIN]) {
-		int i;
+	/* print backtrace */
+	for (i = 0; i < (int)ARRAY_SIZE(sigsegv_codes); i++) {
+		if (sig != SIGSEGV)
+			break;
 
-		for (i = 0; i < (int)ARRAY_SIZE(sigsegv_codes); i++) {
-			if (sig != SIGSEGV)
-				break;
-
-			if (si->si_code == sigsegv_codes[i].code) {
-				pr_red("Segmentation fault: %s (addr: %p)\n",
-				       sigsegv_codes[i].msg, si->si_addr);
-				break;
-			}
-		}
-		if (sig != SIGSEGV || i == (int)ARRAY_SIZE(sigsegv_codes)) {
-			pr_red("process crashed by signal %d: %s (si_code: %d)\n",
-			       sig, strsignal(sig), si->si_code);
+		if (si->si_code == sigsegv_codes[i].code) {
+			pr_warn("Segmentation fault: %s (addr: %p)\n",
+			       sigsegv_codes[i].msg, si->si_addr);
+			break;
 		}
+	}
+	if (sig != SIGSEGV || i == (int)ARRAY_SIZE(sigsegv_codes)) {
+		pr_warn("process crashed by signal %d: %s (si_code: %d)\n",
+		       sig, strsignal(sig), si->si_code);
+	}
 
-		pr_red("Backtrace from uftrace:\n");
-		pr_red("=====================================\n");
+	pr_warn("Backtrace from uftrace:\n");
+	pr_warn("=====================================\n");
 
-		while (rstack >= mtdp->rstack) {
-			struct sym *parent, *child;
-			char *pname, *cname;
+	while (rstack >= mtdp->rstack) {
+		struct sym *parent, *child;
+		char *pname, *cname;
 
-			parent = find_symtabs(&symtabs, rstack->parent_ip);
-			pname = symbol_getname(parent, rstack->parent_ip);
-			child  = find_symtabs(&symtabs, rstack->child_ip);
-			cname = symbol_getname(child, rstack->child_ip);
+		parent = find_symtabs(&symtabs, rstack->parent_ip);
+		pname = symbol_getname(parent, rstack->parent_ip);
+		child  = find_symtabs(&symtabs, rstack->child_ip);
+		cname = symbol_getname(child, rstack->child_ip);
 
-			pr_red("[%d] (%s[%lx] <= %s[%lx])\n", idx--,
-			       cname, rstack->child_ip, pname, rstack->parent_ip);
+		pr_warn("[%d] (%s[%lx] <= %s[%lx])\n", idx--,
+		       cname, rstack->child_ip, pname, rstack->parent_ip);
 
-			symbol_putname(parent, pname);
-			symbol_putname(child, cname);
+		symbol_putname(parent, pname);
+		symbol_putname(child, cname);
 
-			rstack--;
-		}
+		rstack--;
 	}
 
 out:
@@ -814,14 +788,20 @@ static void mcount_finish(void)
 static bool mcount_check_rstack(struct mcount_thread_data *mtdp)
 {
 	if (mtdp->idx >= mcount_rstack_max) {
-		static bool warned = false;
+		if (!mtdp->warned) {
+			struct mcount_ret_stack *rstack;
 
-		if (!warned) {
-			pr_warn("too deeply nested calls: %d\n", mtdp->idx);
-			warned = true;
+			pr_warn("call depth beyond %d is not recorded.\n"
+			        "      (use --max-stack=DEPTH to record more)\n",
+				mtdp->idx);
+			/* flush current rstack */
+			rstack = &mtdp->rstack[mcount_rstack_max - 1];
+			record_trace_data(mtdp, rstack, NULL);
+			mtdp->warned = true;
 		}
 		return true;
 	}
+	mtdp->warned = false;
 	return false;
 }
 
@@ -1026,7 +1006,7 @@ void mcount_entry_filter_record(struct mcount_thread_data *mtdp,
 			 * Flush existing rstack when mcount_enabled is off
 			 * (i.e. disabled).  Note that changing to enabled is
 			 * already handled in record_trace_data() on exit path
-			 * using the MCOUNT_FL_DISALBED flag.
+			 * using the MCOUNT_FL_DISABLED flag.
 			 */
 			if (unlikely(mtdp->enable_cached))
 				record_trace_data(mtdp, rstack, NULL);
@@ -1294,7 +1274,7 @@ static unsigned long __mcount_exit(long *retval)
 	assert(!mtdp->dead);
 
 	/*
-	 * it's only called when mcount_entry() was succeeded
+	 * it's only called when mcount_entry() was succeeded and
 	 * no need to check recursion here.  But still needs to
 	 * prevent recursion during this call.
 	 */
@@ -1318,8 +1298,8 @@ static unsigned long __mcount_exit(long *retval)
 		mtd_dtor(mtdp);
 		/*
 		 * mtd_dtor() will free rstack but current ret_addr
-		 * might be plthook_return() when it was a tailcall.
-		 * reload the return address after mtd_dtor() restored
+		 * might be plthook_return() when it was a tail call.
+		 * Reload the return address after mtd_dtor() restored
 		 * all the parent locations.
 		 */
 		retaddr = *ret_loc;
@@ -1441,7 +1421,7 @@ static void __cygprof_exit(unsigned long parent, unsigned long child)
 
 	/*
 	 * cygprof_exit() can be called beyond rstack max.
-	 * it cannot use mcount_check_rstack() here
+	 * It cannot use mcount_check_rstack() here
 	 * since we didn't decrease the idx yet.
 	 */
 	if (mtdp->idx > mcount_rstack_max)
@@ -1559,7 +1539,7 @@ static void _xray_exit(long *retval)
 
 	/*
 	 * cygprof_exit() can be called beyond rstack max.
-	 * it cannot use mcount_check_rstack() here
+	 * It cannot use mcount_check_rstack() here
 	 * since we didn't decrease the idx yet.
 	 */
 	if (mtdp->idx > mcount_rstack_max)
@@ -1839,6 +1819,7 @@ static void mcount_cleanup(void)
 
 	if (SCRIPT_ENABLED && script_str)
 		script_finish();
+	script_str = NULL;
 
 	unload_module_symtabs();
 
