@@ -243,28 +243,26 @@ static int find_got(struct uftrace_elf_data *elf,
 	mcount_arch_plthook_setup(pd, elf);
 	list_add_tail(&pd->list, &plthook_modules);
 
-	if (plt_found) {
-		if (plthook_resolver_addr == 0)
-			plthook_resolver_addr = pd->pltgot_ptr[2];
+	if (plthook_resolver_addr == 0)
+		plthook_resolver_addr = pd->pltgot_ptr[2];
 
-		/*
-		 * BIND_NOW (+ RELRO) makes module id not used and resets to 0.
-		 * but we still need it to find pd from plthook_enter().
-		 */
-		if (pd->module_id == 0) {
-			pr_dbg2("update module id to %p\n", pd);
-			overwrite_pltgot(pd, 1, pd);
-			pd->module_id = (unsigned long)pd;
-		}
-
-		pr_dbg2("found GOT at %p (base_addr + %#lx)\n", pd->pltgot_ptr,
-			(unsigned long)pd->pltgot_ptr - pd->base_addr);
-		pr_dbg2("module id = %#lx, PLT resolver = %#lx\n",
-			pd->module_id, plthook_resolver_addr);
-
-		restore_plt_functions(pd);
+	/*
+	 * BIND_NOW (+ RELRO) makes module id not used and resets to 0.
+	 * but we still need it to find pd from plthook_enter().
+	 */
+	if (pd->module_id == 0) {
+		pr_dbg2("update module id to %p\n", pd);
+		overwrite_pltgot(pd, 1, pd);
+		pd->module_id = (unsigned long)pd;
 	}
 
+	pr_dbg2("found GOT at %p (base_addr + %#lx)\n", pd->pltgot_ptr,
+		(unsigned long)pd->pltgot_ptr - pd->base_addr);
+	pr_dbg2("module id = %#lx, PLT resolver = %#lx\n",
+		pd->module_id, plthook_resolver_addr);
+
+	restore_plt_functions(pd);
+
 	overwrite_pltgot(pd, 2, plt_hooker);
 
 	if (getenv("LD_BIND_NOT"))
@@ -644,12 +642,12 @@ static void setup_vfork(struct mcount_thread_data *mtdp)
 	update_kernel_tid(tmsg.tid);
 }
 
-/* this function detects whether child finished */
+/* this function detects whether child is finished */
 static struct mcount_ret_stack * restore_vfork(struct mcount_thread_data *mtdp,
 					       struct mcount_ret_stack *rstack)
 {
 	/*
-	 * On vfork, parent sleeps until child exec'ed or exited.
+	 * On vfork, parent sleeps until child is exec'ed or exited.
 	 * So if it sees parent pid, that means child was done.
 	 */
 	if (getpid() == vfork_parent) {
@@ -920,7 +918,7 @@ static unsigned long __plthook_exit(long *retval)
 	assert(!check_thread_data(mtdp));
 
 	/*
-	 * it's only called when mcount_entry() was succeeded
+	 * it's only called when mcount_entry() was succeeded and
 	 * no need to check recursion here.  But still needs to
 	 * prevent recursion during this call.
 	 */
@@ -997,8 +995,8 @@ again:
 		mtd_dtor(mtdp);
 		/*
 		 * mtd_dtor() will free rstack but current ret_addr
-		 * might be plthook_return() when it was a tailcall.
-		 * reload the return address after mtd_dtor() restored
+		 * might be plthook_return() when it was a tail call.
+		 * Reload the return address after mtd_dtor() restored
 		 * all the parent locations.
 		 */
 		if (ARCH_CAN_RESTORE_PLTHOOK)
