@@ -131,7 +131,7 @@ static void get_new_shmem_buffer(struct mcount_thread_data *mtdp)
 
 reuse:
 	/*
-	 * Start a new buffer and mark it recording data.
+	 * Start a new buffer and mark its recording data.
 	 * See cmd-record.c::writer_thread().
 	 */
 	__sync_fetch_and_or(&curr_buf->flag, SHMEM_FL_RECORDING);
@@ -363,8 +363,8 @@ static bool find_mem_region(struct rb_root *root, unsigned long addr)
 	return false;
 }
 
-static bool check_mem_region(struct mcount_arg_context *ctx,
-			     unsigned long addr)
+bool check_mem_region(struct mcount_arg_context *ctx,
+		      unsigned long addr)
 {
 	bool update = true;
 	struct mcount_mem_regions *regions = ctx->regions;
@@ -433,9 +433,11 @@ static unsigned save_to_argbuf(void *argbuf, struct list_head *args_spec,
 				 */
 				long *base = ctx->val.p;
 				long *_M_string_length = base + 1;
-				char *_M_dataplus = (char*)(*base);
-				len = *_M_string_length;
-				str = _M_dataplus;
+				if (check_mem_region(ctx, (unsigned long)base)) {
+					char *_M_dataplus = (char*)(*base);
+					len = *_M_string_length;
+					str = _M_dataplus;
+				}
 			}
 
 			if (str) {
@@ -443,7 +445,7 @@ static unsigned save_to_argbuf(void *argbuf, struct list_head *args_spec,
 				char *dst = ptr + 2;
 				char buf[32];
 
-				if (!check_mem_region(ctx, ctx->val.i)) {
+				if (!check_mem_region(ctx, (unsigned long)str)) {
 					len = snprintf(buf, sizeof(buf), "<%p>", str);
 					str = buf;
 				}
@@ -540,7 +542,7 @@ void save_retval(struct mcount_thread_data *mtdp,
 	*(uint32_t *)argbuf = size;
 }
 
-static int save_proc_statm(void *buf)
+static int save_proc_statm(void *ctx, void *buf)
 {
 	FILE *fp;
 	struct uftrace_proc_statm *statm = buf;
@@ -552,9 +554,11 @@ static int save_proc_statm(void *buf)
 	if (fscanf(fp, "%"SCNu64" %"SCNu64" %"SCNu64,
 		   &statm->vmsize, &statm->vmrss, &statm->shared) != 3)
 		pr_err("failed to scan /proc/self/statm");
-
-	/* Since /proc/[pid]/statm prints the number of pages for each field,
-	 * it'd be better to keep the memory size in KB. */
+		
+	/* 
+	 * Since /proc/[pid]/statm prints the number of pages for each field,
+	 * it'd be better to keep the memory size in KB.
+	 */
 	statm->vmsize *= page_size_in_kb;
 	statm->vmrss  *= page_size_in_kb;
 	statm->shared *= page_size_in_kb;
@@ -563,7 +567,7 @@ static int save_proc_statm(void *buf)
 	return 0;
 }
 
-static void diff_proc_statm(void *dst, void *src)
+static void diff_proc_statm(void *ctx, void *dst, void *src)
 {
 	struct uftrace_proc_statm *dst_statm = dst;
 	struct uftrace_proc_statm *src_statm = src;
@@ -573,7 +577,7 @@ static void diff_proc_statm(void *dst, void *src)
 	dst_statm->shared -= src_statm->shared;
 }
 
-static int save_page_fault(void *buf)
+static int save_page_fault(void *ctx, void *buf)
 {
 	struct rusage ru;
 	struct uftrace_page_fault *page_fault = buf;
@@ -587,7 +591,7 @@ static int save_page_fault(void *buf)
 	return 0;
 }
 
-static void diff_page_fault(void *dst, void *src)
+static void diff_page_fault(void *ctx, void *dst, void *src)
 {
 	struct uftrace_page_fault *dst_pgflt = dst;
 	struct uftrace_page_fault *src_pgflt = src;
@@ -596,46 +600,52 @@ static void diff_page_fault(void *dst, void *src)
 	dst_pgflt->minor -= src_pgflt->minor;
 }
 
-static int save_pmu_cycle(void *buf)
+static int save_pmu_cycle(void *ctx, void *buf)
 {
-	return read_pmu_event(EVENT_ID_READ_PMU_CYCLE, buf);
+	return read_pmu_event(ctx, EVENT_ID_READ_PMU_CYCLE, buf);
 }
 
-static void diff_pmu_cycle(void *dst, void *src)
+static void diff_pmu_cycle(void *ctx, void *dst, void *src)
 {
 	struct uftrace_pmu_cycle *dst_cycle = dst;
 	struct uftrace_pmu_cycle *src_cycle = src;
 
 	dst_cycle->cycles -= src_cycle->cycles;
 	dst_cycle->instrs -= src_cycle->instrs;
+
+	release_pmu_event(ctx, EVENT_ID_READ_PMU_CYCLE);
 }
 
-static int save_pmu_cache(void *buf)
+static int save_pmu_cache(void *ctx, void *buf)
 {
-	return read_pmu_event(EVENT_ID_READ_PMU_CACHE, buf);
+	return read_pmu_event(ctx, EVENT_ID_READ_PMU_CACHE, buf);
 }
 
-static void diff_pmu_cache(void *dst, void *src)
+static void diff_pmu_cache(void *ctx, void *dst, void *src)
 {
 	struct uftrace_pmu_cache *dst_cache = dst;
 	struct uftrace_pmu_cache *src_cache = src;
 
 	dst_cache->refers -= src_cache->refers;
 	dst_cache->misses -= src_cache->misses;
+
+	release_pmu_event(ctx, EVENT_ID_READ_PMU_CACHE);
 }
 
-static int save_pmu_branch(void *buf)
+static int save_pmu_branch(void *ctx, void *buf)
 {
-	return read_pmu_event(EVENT_ID_READ_PMU_BRANCH, buf);
+	return read_pmu_event(ctx, EVENT_ID_READ_PMU_BRANCH, buf);
 }
 
-static void diff_pmu_branch(void *dst, void *src)
+static void diff_pmu_branch(void *ctx, void *dst, void *src)
 {
 	struct uftrace_pmu_branch *dst_branch = dst;
 	struct uftrace_pmu_branch *src_branch = src;
 
 	dst_branch->branch -= src_branch->branch;
 	dst_branch->misses -= src_branch->misses;
+
+	release_pmu_event(ctx, EVENT_ID_READ_PMU_BRANCH);
 }
 
 /* above functions should follow the name convention to use below macro */
@@ -648,8 +658,8 @@ static struct read_event_data {
 	enum uftrace_event_id	id_read;
 	enum uftrace_event_id	id_diff;
 	size_t			size;
-	int (*save)(void *buf);
-	void (*diff)(void *dst, void *src);
+	int (*save)(void *ctx, void *buf);
+	void (*diff)(void *ctx, void *dst, void *src);
 } read_events[] = {
 	{ TR_ID(PROC_STATM), TR_DS(proc_statm), TR_FN(proc_statm) },
 	{ TR_ID(PAGE_FAULT), TR_DS(page_fault), TR_FN(page_fault) },
@@ -694,7 +704,7 @@ void save_trigger_read(struct mcount_thread_data *mtdp,
 		event->dsize = red->size;
 		event->idx   = mtdp->idx;
 
-		if (red->save(event->data) < 0)
+		if (red->save(mtdp, event->data) < 0)
 			continue;
 
 		if (diff) {
@@ -705,11 +715,13 @@ void save_trigger_read(struct mcount_thread_data *mtdp,
 				old_event = get_event_pointer(ptr, idx);
 				if (old_event->id == event->id)
 					break;
+
+				old_event = NULL;
 			}
 
 			if (old_event) {
 				event->id = red->id_diff;
-				red->diff(event->data, old_event->data);
+				red->diff(mtdp, event->data, old_event->data);
 			}
 		}
 
@@ -737,8 +749,8 @@ void save_watchpoint(struct mcount_thread_data *mtdp,
 		 * Normally watch point event comes before the rstack (record)
 		 * in order to indicate where it's changed precisely.
 		 * But first watch point event needs to come after the first
-		 * record otherwise it'd not shown since 'event-skip' mechanism.
-		 * so add 2(nsec) so that it can be 1 nsec later.
+		 * record otherwise it'd not show since 'event-skip' mechanism.
+		 * Therefore, add 2(nsec) so that it can be 1 nsec later.
 		 */
 		timestamp += 2;
 		mtdp->watch.inited = true;
@@ -789,6 +801,12 @@ void save_watchpoint(struct mcount_thread_data *mtdp,
 		     unsigned long watchpoints)
 {
 }
+
+bool check_mem_region(struct mcount_arg_context *ctx,
+		      unsigned long addr)
+{
+	return true;
+}
 #endif
 
 static struct mcount_shmem_buffer * get_shmem_buffer(struct mcount_thread_data *mtdp,
@@ -837,8 +855,8 @@ static int record_event(struct mcount_thread_data *mtdp,
 	rec = (void *)(curr_buf->data + curr_buf->size);
 
 	/*
-	 * instead of set bitfields, do the bit operations manually.
-	 * this would be good both for performance and portability.
+	 * instead of set bit fields, do the bit operations manually.
+	 * this would be good for both performance and portability.
 	 */
 	rec->data  = UFTRACE_EVENT | RECORD_MAGIC << 3;
 	rec->data += (uint64_t)event->id << 16;
@@ -928,8 +946,8 @@ static int record_ret_stack(struct mcount_thread_data *mtdp,
 	frstack->addr   = mrstack->child_ip;
 #else
 	/*
-	 * instead of set bitfields, do the bit operations manually.
-	 * this would be good both for performance and portability.
+	 * instead of set bit fields, do the bit operations manually.
+	 * this would be good for both performance and portability.
 	 */
 	rec  = type | RECORD_MAGIC << 3;
 	rec += argbuf ? 4 : 0;
@@ -1159,8 +1177,9 @@ void record_proc_maps(char *dirname, const char *sess_id,
 		map->libname[strlen(path)] = '\0';
 
 		/* still need to write the map for executable */
-		if (!strcmp(path, symtabs->filename))
-			symtabs->exec_base = start;
+		if (!strcmp(path, symtabs->filename)) {
+			symtabs->exec_map = map;
+		}
 
 		if (prev_map)
 			prev_map->next = map;
