@@ -371,6 +371,65 @@ class TestBase:
 
         return True
 
+    def is_32bit(self):
+        EI_NIDENT = 16
+
+        # e_ident[] indexes
+        EI_CLASS      = 4
+
+        # EI_CLASS
+        ELFCLASSNONE  = 0
+        ELFCLASS32    = 1
+        ELFCLASS64    = 2
+        ELFCLASSNUM   = 3
+
+        try:
+            elfname = 't-' + self.name
+            f = open(elfname, 'rb')
+            elf_ident = list(f.read(EI_NIDENT))
+            ei_class = ord(elf_ident[EI_CLASS])
+            f.close()
+
+            if ei_class == ELFCLASS32:
+                return True
+        except:
+            pass
+
+        return False
+
+    def get_elf_machine(self):
+        EI_NIDENT = 16
+
+        # e_machine (architecture)
+        EM_386        = 3       # Intel 80386
+        EM_ARM        = 40      # ARM
+        EM_X86_64     = 62      # AMD x86-64 architecture
+        EM_AARCH64    = 183     # ARM AARCH64
+
+        machine = {
+            EM_386: 'i386',
+            EM_ARM: 'arm',
+            EM_X86_64: 'x86_64',
+            EM_AARCH64: 'aarch64'
+        }
+
+        try:
+            elfname = 't-' + self.name
+            f = open(elfname, 'rb')
+
+            # consume elf_ident and e_type
+            f.read(EI_NIDENT + 2)
+
+            # read e_machine
+            e_machine = ord(f.read(2)[0])
+            f.close()
+
+            return machine[e_machine]
+        except:
+            pass
+
+        return None
+
     def prerun(self, timeout):
         ret = TestBase.TEST_SUCCESS
 
@@ -559,11 +618,13 @@ def save_test_result(result, case, shared):
         shared.total += 1
 
 
-def print_test_result(case, result, diffs, color):
+def print_test_result(case, result, diffs, color, ftests):
+    plain_result = [text_result[r] for r in result]
+
     if color:
         result_list = [colored_result[r] for r in result]
     else:
-        result_list = [text_result[r] for r in result]
+        result_list = plain_result
 
     for dif in diffs:
         if dif != '':
@@ -571,10 +632,21 @@ def print_test_result(case, result, diffs, color):
 
     output = case[1:4]
     output += ' %-20s' % case[5:] + ': ' + ' '.join(result_list) + '\n'
+
     sys.stdout.write(output)
 
+    # write abnormal test result to failed-tests.txt
+    normal = [TestBase.TEST_SUCCESS, TestBase.TEST_SUCCESS_FIXED, TestBase.TEST_SKIP]
+    for r in result:
+        if r not in normal:
+            output = case[1:4]
+            output += ' %-20s' % case[5:] + ': ' + ' '.join(plain_result) + '\n'
+            ftests.write(output)
+            ftests.flush()
+            break
 
-def print_test_header(opts, flags):
+
+def print_test_header(opts, flags, ftests):
     optslen = len(opts)
 
     header1 = '%-24s ' % 'Test case'
@@ -588,6 +660,9 @@ def print_test_header(opts, flags):
 
     print(header1)
     print(header2)
+    ftests.write(header1 + '\n')
+    ftests.write(header2 + '\n')
+    ftests.flush()
 
 
 def print_test_report(color, shared):
@@ -646,6 +721,7 @@ if __name__ == "__main__":
     else:
         try:
             testcases = glob.glob('t*' + arg.case + '*.py')
+            arg.worker = min(arg.worker, len(testcases))
         finally:
             if len(testcases) == 0:
                 print("cannot find testcase for : %s" % arg.case)
@@ -681,6 +757,8 @@ if __name__ == "__main__":
     res.append(TestBase.TEST_UNSUPP_LANG)
     res.append(TestBase.TEST_SKIP)
 
+    ftests = open("failed-tests.txt", "w")
+
     shared.stats = dict.fromkeys(res, 0)
     pool = multiprocessing.Pool(arg.worker)
     serial_pool = multiprocessing.Pool(1)
@@ -696,7 +774,7 @@ if __name__ == "__main__":
                          args=[name, flags, opts.split(), arg])
 
     print("Start %s tests with %d worker" % (shared.tests_count, arg.worker))
-    print_test_header(opts, flags)
+    print_test_header(opts, flags, ftests)
 
     color = arg.color
     if not sys.stdout.isatty():
@@ -710,11 +788,13 @@ if __name__ == "__main__":
         while name not in shared.results:
             time.sleep(1)
 
-        print_test_result(name, shared.results[name], shared.diffs[name], color)
+        print_test_result(name, shared.results[name], shared.diffs[name], color, ftests)
 
     pool.close()
     pool.join()
 
+    ftests.close()
+
     sys.stdout.write("\n")
     sys.stdout.flush()
 
