@@ -8,7 +8,7 @@ class TestCase(TestBase):
 # DURATION    TID     FUNCTION
             [18279] | main() {
    0.371 ms [18279] |   foo('f', 'o', 'o');
-   0.923 ms [18279] |   bar('\\x00', 'B', 97, 0x72);
+   0.923 ms [18279] |   bar('\\0', 'B', 97, 0x72);
    3.281 ms [18279] | } /* main */
 """)
 
