@@ -26,8 +26,7 @@ class TestCase(TestBase):
     def runcmd(self):
         argopt = '-A "^int_@arg1,arg2" -R "^int_@retval/i32"'
 
-        import platform
-        if platform.architecture()[0].startswith('32bit'):
+        if TestBase.is_32bit(self):
             # int_mul@arg1 is a 'long long', so we should skip arg2
             argopt  = '-A "int_(add|sub|div)@arg1,arg2" -A "int_mul@arg1/i64,arg3" '
             argopt += '-R "^int_@retval/i32"'
