@@ -1,6 +1,5 @@
 #!/usr/bin/env python
 
-import re
 from runtest import TestBase
 
 class TestCase(TestBase):
@@ -18,5 +17,5 @@ class TestCase(TestBase):
 """)
 
     def runcmd(self):
-        return '%s -t 1ms -T "mem_alloc@trace" %s' % (TestBase.ftrace, 't-' + self.name)
+        return '%s -t 1ms -T "mem_alloc@trace" %s' % (TestBase.uftrace_cmd, 't-' + self.name)
 
