@@ -36,4 +36,4 @@ class TestCase(TestBase):
         argopt  = '-A "many@arg1/i32,arg2/i32,arg3/i32,arg4/i32,'
         argopt += 'arg5/i32,arg6/i32,arg7/i32,arg8/i32,arg9/i32,'
         argopt += 'arg10/i32,arg11/i32,arg12/i32,arg13/i32"'
-        return '%s %s %s' % (TestBase.ftrace, argopt, 't-' + self.name)
+        return '%s %s %s' % (TestBase.uftrace_cmd, argopt, 't-' + self.name)
