@@ -15,6 +15,11 @@ class TestCase(TestBase):
    4.891 ms [18277] | } /* main */
 """)
 
+    def pre(self):
+        if TestBase.get_elf_machine(self) == 'i386':
+            return TestBase.TEST_SKIP
+        return TestBase.TEST_SUCCESS
+
     def build(self, name, cflags='', ldflags=''):
         # cygprof doesn't support arguments now
         if cflags.find('-finstrument-functions') >= 0:
@@ -29,8 +34,7 @@ class TestCase(TestBase):
         argopt += '-A "mixed_div@arg1/i64,fparg1/80%stack+1" '
         argopt += '-A "mixed_str@arg1/s%rdi,fparg1%xmm0"'
 
-        import platform
-        if platform.machine().startswith('arm'):
+        if TestBase.get_elf_machine(self) == 'arm':
             argopt = argopt.replace('%rdi', '%r0')
             argopt = argopt.replace('%rsi', '%r1')
             argopt = argopt.replace('32%xmm0', '32%s0')
