@@ -58,9 +58,7 @@ reading 5188.dat
         return ret
 
     def fixup(self, cflags, result):
-        import platform
-
-        if platform.architecture()[0] == '32bit':
+        if TestBase.is_32bit(self):
             result = result.replace("2 (64 bit)", "1 (32 bit)")
         p = sp.Popen(['file', 't-' + self.name], stdout=sp.PIPE)
         if 'BuildID' not in p.communicate()[0].decode(errors='ignore'):
