@@ -2,14 +2,14 @@
 
 from runtest import TestBase
 import subprocess as sp
-import os, re
+import os
 
 TDIR='xxx'
 FUNC='main'
 
 class TestCase(TestBase):
     def __init__(self):
-        TestBase.__init__(self, 'getids', result="""
+        TestBase.__init__(self, 'getids', serial=True, result="""
 # Function Call Graph for 'main' (session: 59268c360e3c1bd6)
 =============== BACKTRACE ===============
  backtrace #0: hit 1, time  24.837 us
@@ -73,6 +73,6 @@ class TestCase(TestBase):
         major, minor, release = uname[2].split('.')
         if uname[0] == 'Linux' and uname[4] == 'x86_64' and \
            int(major) >= 4 and int(minor) >= 17:
-            result = re.sub('sys_[a-zA-Z0-9_]+', 'do_syscall_64', result)
+            result = result.replace('sys_get', '__x64_sys_get')
 
         return result
