@@ -17,7 +17,7 @@ class TestCase(TestBase):
 """)
 
     def runcmd(self):
-        return '%s -F main --auto-args %s hello' % (TestBase.ftrace, 't-' + self.name)
+        return '%s -F main --auto-args %s hello' % (TestBase.uftrace_cmd, 't-' + self.name)
 
     def sort(self, output):
         result = []
