@@ -9,11 +9,11 @@ class TestCase(TestBase):
 # DURATION    TID     FUNCTION
             [14885] | main() {
             [14885] |   foo() {
-   9.397 us [14885] |     open("/dev/zero", O_RDONLY) = 4;
-   3.802 us [14885] |     mmap(0, 4096, PROT_READ, MAP_ANON|MAP_PRIVATE, 4, 0) = 0xADDR;
+   9.397 us [14885] |     open("/dev/zero", O_RDONLY) = 5;
+   3.802 us [14885] |     mmap(0, 4096, PROT_READ, MAP_ANON|MAP_PRIVATE, 5, 0) = 0xADDR;
    4.119 us [14885] |     mprotect(0xADDR, 4096, PROT_NONE) = 0;
    6.183 us [14885] |     munmap(0xADDR, 4096) = 0;
-   3.120 us [14885] |     close(4) = 0;
+   3.120 us [14885] |     close(5) = 0;
   36.529 us [14885] |   } /* foo */
   37.849 us [14885] | } /* main */
 """)
@@ -32,3 +32,6 @@ class TestCase(TestBase):
             result.append(func)
 
         return '\n'.join(result)
+
+    def fixup(self, cflags, result):
+        return result.replace('5', '4')
