@@ -23,6 +23,9 @@ class TestCase(TestBase):
 """)
 
     def pre(self):
+        if not TestBase.check_perf_paranoid(self):
+            return TestBase.TEST_SKIP
+
         uftrace  = TestBase.uftrace_cmd
         argument = '-d %s -E linux:task-name' % TDIR
         program  = 't-' + self.name
