@@ -23,6 +23,11 @@ class TestCase(TestBase):
              bar | } /* main */
 """, sort='simple')
 
+    def pre(self):
+        if not TestBase.check_perf_paranoid(self):
+            return TestBase.TEST_SKIP
+        return TestBase.TEST_SUCCESS
+
     def runcmd(self):
         uftrace  = TestBase.uftrace_cmd
         options = '-F main -E linux:task-name -t 1'
