@@ -112,7 +112,7 @@ next:
 		while (!list_empty(&args)) {
 			arg = list_first_entry(&args, struct uftrace_arg_spec, list);
 			list_del(&arg->list);
-			free(arg);
+			free_arg_spec(arg);
 		}
 	}
 	strv_free(&specs);
@@ -261,11 +261,11 @@ static void release_auto_args(struct rb_root *root)
 
 		list_for_each_entry_safe(arg, tmp, &entry->args, list) {
 			list_del(&arg->list);
-			free(arg);
+			free_arg_spec(arg);
 		}
 
 		free(entry->name);
-		free((void *)entry->end);
+		free((void *)(uintptr_t)entry->end);
 		free(entry);
 	}
 }
@@ -286,7 +286,7 @@ void finish_auto_args(void)
 			spec = list_first_entry(dwarf_argspec_list->trigger.pargs,
 						typeof(*spec), list);
 			list_del(&spec->list);
-			free(spec);
+			free_arg_spec(spec);
 		}
 		free(dwarf_argspec_list);
 
@@ -346,6 +346,7 @@ int extract_trigger_args(char **pargs, char **prets, char *trigger)
 				xasprintf(&act, "%s@%s", name, args);
 				argspec = strjoin(argspec, act, ";");
 				free(act);
+				free(args);
 			}
 			if (rval) {
 				xasprintf(&act, "%s@retval", name);
