@@ -122,18 +122,18 @@ int uftrace_count_filter(struct rb_root *root, unsigned long flag)
 	return count;
 }
 
-static bool match_ip(struct uftrace_filter *filter, unsigned long ip)
+static bool match_ip(struct uftrace_filter *filter, uint64_t addr)
 {
-	return filter->start <= ip && ip < filter->end;
+	return filter->start <= addr && addr < filter->end;
 }
 
 /**
  * uftrace_match_filter - try to match @ip with filters in @root
- * @ip   - instruction address to match
+ * @addr - instruction address to match
  * @root - root of rbtree which has filters
  * @tr   - trigger data
  */
-struct uftrace_filter *uftrace_match_filter(uint64_t ip, struct rb_root *root,
+struct uftrace_filter *uftrace_match_filter(uint64_t addr, struct rb_root *root,
 					   struct uftrace_trigger *tr)
 {
 	struct rb_node *parent = NULL;
@@ -144,8 +144,8 @@ struct uftrace_filter *uftrace_match_filter(uint64_t ip, struct rb_root *root,
 		parent = *p;
 		iter = rb_entry(parent, struct uftrace_filter, node);
 
-		if (match_ip(iter, ip)) {
-		  	*tr = iter->trigger;
+		if (match_ip(iter, addr)) {
+			*tr = iter->trigger;
 
 			pr_dbg2("filter match: %s\n", iter->name);
 			if (dbg_domain[DBG_FILTER] >= 3)
@@ -153,7 +153,7 @@ struct uftrace_filter *uftrace_match_filter(uint64_t ip, struct rb_root *root,
 			return iter;
 		}
 
-		if (iter->start > ip)
+		if (iter->start > addr)
 			p = &parent->rb_left;
 		else
 			p = &parent->rb_right;
@@ -220,6 +220,13 @@ static void add_arg_spec(struct list_head *arg_list, struct uftrace_arg_spec *ar
 	}
 }
 
+void free_arg_spec(struct uftrace_arg_spec *arg)
+{
+	if (arg->fmt == ARG_FMT_ENUM)
+		free(arg->enum_str);
+	free(arg);
+}
+
 void add_trigger(struct uftrace_filter *filter, struct uftrace_trigger *tr,
 		 bool exact_match)
 {
@@ -997,7 +1004,7 @@ static void setup_trigger(char *filter_str, struct symtabs *symtabs,
 			if (!strcasecmp(module, "PLT")) {
 				setting->plt_only = true;
 				ret += add_trigger_entry(root, &patt, &tr,
-							 symtabs->maps,
+							 symtabs->exec_map,
 							 setting);
 				setting->plt_only = false;
 			}
@@ -1043,10 +1050,7 @@ next:
 		while (!list_empty(&args)) {
 			arg = list_first_entry(&args, typeof(*arg), list);
 			list_del(&arg->list);
-
-			if (arg->fmt == ARG_FMT_ENUM)
-				free(arg->enum_str);
-			free(arg);
+			free_arg_spec(arg);
 		}
 
 	}
@@ -1156,7 +1160,7 @@ void uftrace_cleanup_filter(struct rb_root *root)
 
 		list_for_each_entry_safe(arg, tmp, &filter->args, list) {
 			list_del(&arg->list);
-			free(arg);
+			free_arg_spec(arg);
 		}
 		free(filter);
 	}
@@ -1235,6 +1239,7 @@ static void filter_test_load_symtabs(struct symtabs *stabs)
 	mod.symtab.nr_sym = ARRAY_SIZE(syms);
 
 	stabs->maps = &map;
+	stabs->exec_map = &map;
 	stabs->loaded = true;
 	stabs->loaded_debug = true;  /* skip DWARF debug info parsing */
 }
