@@ -109,8 +109,8 @@ struct uftrace_trigger {
 struct uftrace_filter {
 	struct rb_node		node;
 	char 			*name;
-	unsigned long		start;
-	unsigned long		end;
+	uint64_t		start;
+	uint64_t		end;
 	struct list_head	args;
 	struct uftrace_trigger	trigger;
 };
@@ -206,6 +206,8 @@ void setup_auto_args_str(char *args, char *rets, char *enums,
 			 struct uftrace_filter_setting *setting);
 void finish_auto_args(void);
 
+void free_arg_spec(struct uftrace_arg_spec *arg);
+
 struct debug_info;
 
 struct uftrace_filter * find_auto_argspec(struct uftrace_filter *filter,
