@@ -237,8 +237,9 @@ out:
  * @handle - uftrace data file handle
  *
  * This function prepares to read perf event data from perf-cpu*.dat
- * files.  It returns 0 on success, -1 on failure.  Callers should
- * call finish_perf_data() after reading all perf event data.
+ * files.  It returns 0 on success which includes that perf event data
+ * already setup, -1 on failure.  Callers should call
+ * finish_perf_data() after reading all perf event data.
  */
 int setup_perf_data(struct uftrace_data *handle)
 {
@@ -248,6 +249,9 @@ int setup_perf_data(struct uftrace_data *handle)
 	size_t i;
 	int ret = -1;
 
+	if (has_perf_data(handle))
+		return 0;
+
 	xasprintf(&pattern, "%s/perf-cpu*.dat", handle->dirname);
 	if (glob(pattern, GLOB_ERR, NULL, &globbuf)) {
 		pr_dbg("failed to search perf data file\n");
@@ -510,13 +514,17 @@ void update_perf_task_comm(struct uftrace_data *handle)
 			if (read_perf_event(handle, perf) < 0)
 				continue;
 
-			if (perf->type != PERF_RECORD_COMM)
-				continue;
-
 			task = find_task(&handle->sessions, perf->tid);
 			if (task == NULL)
 				continue;
 
+			if (task->time.stamp == 0 ||
+			    task->time.stamp > perf->time)
+				task->time.stamp = perf->time;
+
+			if (perf->type != PERF_RECORD_COMM)
+				continue;
+
 			memcpy(task->comm, perf->u.comm.comm, sizeof(task->comm));
 		}
 
