@@ -6,7 +6,7 @@
  * Released under the GPL v2.
  */
 
-#ifdef HAVE_LIBPYTHON2
+#if defined(HAVE_LIBPYTHON2) || defined(HAVE_LIBPYTHON3)
 
 /* This should be defined before #include "utils.h" */
 #define PR_FMT     "script"
@@ -21,8 +21,13 @@
 #include "utils/script.h"
 #include "utils/script-python.h"
 
+#ifdef HAVE_LIBPYTHON2
 /* python library name, it only supports python 2.7 as of now */
-static const char *libpython = "libpython2.7.so";
+static const char libpython[] = "libpython2.7.so";
+#else
+/* python library name, it should support any version python3 */
+static const char libpython[] = "libpython" stringify(LIBPYTHON3_VERSION) ".so";
+#endif
 
 /* python library handle returned by dlopen() */
 static void *python_handle;
@@ -33,6 +38,9 @@ static pthread_mutex_t python_interpreter_lock = PTHREAD_MUTEX_INITIALIZER;
 /* whether error in script was reported to user */
 static bool python_error_reported = false;
 
+/* whether script_init() was done successfully */
+static bool python_initialized;
+
 static PyAPI_FUNC(void) (*__Py_Initialize)(void);
 static PyAPI_FUNC(void) (*__Py_Finalize)(void);
 static PyAPI_FUNC(void) (*__PySys_SetPath)(char *);
@@ -99,7 +107,16 @@ static const char *py_context_table[] = {
 	do { \
 		__##func = dlsym(python_handle, #func); \
 		if (!__##func) { \
-			pr_err("dlsym for \"" #func "\" is failed!\n"); \
+			pr_err("dlsym for \"" #func "\" is failed"); \
+			return -1; \
+		} \
+	} while (0)
+
+#define INIT_PY_API_FUNC2(func, name)		\
+	do { \
+		__##func = dlsym(python_handle, #name); \
+		if (!__##func) { \
+			pr_err("dlsym for \"" #name "\" is failed"); \
 			return -1; \
 		} \
 	} while (0)
@@ -111,11 +128,23 @@ static int load_python_api_funcs(void)
 		pr_warn("%s cannot be loaded!\n", libpython);
 		return -1;
 	}
+	pr_dbg("%s is loaded\n", libpython);
 
 	INIT_PY_API_FUNC(Py_Initialize);
-	INIT_PY_API_FUNC(Py_Finalize);
-	INIT_PY_API_FUNC(PySys_SetPath);
 	INIT_PY_API_FUNC(PyImport_Import);
+	INIT_PY_API_FUNC(Py_Finalize);
+
+#ifdef HAVE_LIBPYTHON2
+	INIT_PY_API_FUNC(PySys_SetPath);
+	INIT_PY_API_FUNC(PyString_FromString);
+	INIT_PY_API_FUNC(PyInt_FromLong);
+	INIT_PY_API_FUNC(PyString_AsString);
+#else
+	INIT_PY_API_FUNC2(PySys_SetPath, Py_SetPath);
+	INIT_PY_API_FUNC2(PyString_FromString, PyUnicode_FromString);
+	INIT_PY_API_FUNC2(PyInt_FromLong, PyLong_FromLong);
+	INIT_PY_API_FUNC2(PyString_AsString, PyUnicode_AsUTF8);
+#endif
 
 	INIT_PY_API_FUNC(PyErr_Occurred);
 	INIT_PY_API_FUNC(PyErr_Print);
@@ -127,14 +156,11 @@ static int load_python_api_funcs(void)
 	INIT_PY_API_FUNC(PyObject_CallObject);
 	INIT_PY_API_FUNC(PyRun_SimpleStringFlags);
 
-	INIT_PY_API_FUNC(PyString_FromString);
-	INIT_PY_API_FUNC(PyInt_FromLong);
 	INIT_PY_API_FUNC(PyLong_FromLong);
 	INIT_PY_API_FUNC(PyLong_FromUnsignedLongLong);
 	INIT_PY_API_FUNC(PyFloat_FromDouble);
 	INIT_PY_API_FUNC(PyBool_FromLong);
 
-	INIT_PY_API_FUNC(PyString_AsString);
 	INIT_PY_API_FUNC(PyLong_AsLong);
 
 	INIT_PY_API_FUNC(PyTuple_New);
@@ -197,14 +223,14 @@ static int import_python_module(char *py_pathname)
 	free(py_basename);
 
 	pModule = __PyImport_Import(pName);
+	Py_XDECREF(pName);
+
 	if (pModule == NULL) {
 		__PyErr_Print();
 		pr_warn("\"%s\" cannot be imported!\n", py_pathname);
 		return -1;
 	}
 
-	Py_XDECREF(pName);
-
 	/* import sys by default */
 	__PyRun_SimpleStringFlags("import sys", NULL);
 
@@ -340,17 +366,7 @@ static void setup_argument_context(PyObject **pDict, bool is_retval,
 	struct uftrace_arg_spec *spec;
 	void *data = sc_ctx->argbuf;
 	PyObject *args;
-	union {
-		char          c;
-		short         s;
-		int           i;
-		long          l;
-		long long     L;
-		float         f;
-		double        d;
-		long double   D;
-		unsigned char v[16];
-	} val;
+	union script_arg_val val;
 	int count = 0;
 
 	list_for_each_entry(spec, sc_ctx->argspec, list) {
@@ -666,16 +682,23 @@ int script_init_for_python(struct script_info *info,
 	script_uftrace_end = python_uftrace_end;
 	script_atfork_prepare = python_atfork_prepare;
 
-	load_python_api_funcs();
-	set_python_path(py_pathname);
+	if (load_python_api_funcs() < 0)
+		return -1;
+
+	if (set_python_path(py_pathname) < 0) {
+		dlclose(python_handle);
+		return -1;
+	}
 
 	pthread_mutex_lock(&python_interpreter_lock);
 
 	__Py_Initialize();
+	python_initialized = true;
 
 	/* Import python module that is passed by -p option. */
 	if (import_python_module(py_pathname) < 0) {
 		pthread_mutex_unlock(&python_interpreter_lock);
+		/* script_finish() will release resources */
 		return -1;
 	}
 
@@ -715,6 +738,9 @@ void script_finish_for_python(void)
 {
 	pr_dbg("%s()\n", __func__);
 
+	if (!python_initialized)
+		return;
+
 	pthread_mutex_lock(&python_interpreter_lock);
 
 	__Py_Finalize();
@@ -725,4 +751,4 @@ void script_finish_for_python(void)
 	python_handle = NULL;
 }
 
-#endif /* !HAVE_LIBPYTHON2 */
+#endif /* !(HAVE_LIBPYTHON2 || HAVE_LIBPYTHON3) */
