@@ -12,11 +12,11 @@
 
 struct script_info;
 
-#ifdef HAVE_LIBPYTHON2
+#if defined(HAVE_LIBPYTHON2) || defined(HAVE_LIBPYTHON3)
 
-#include <python2.7/Python.h>
+#include <Python.h>
 
-#define SCRIPT_ENABLED 1
+#define SCRIPT_PYTHON_ENABLED 1
 int script_init_for_python(struct script_info *info,
 			   enum uftrace_pattern_type ptype);
 void script_finish_for_python(void);
@@ -26,7 +26,7 @@ void script_finish_for_python(void);
 
 
 /* Do nothing if libpython2.7.so is not installed. */
-#define SCRIPT_ENABLED 0
+#define SCRIPT_PYTHON_ENABLED 0
 static inline int script_init_for_python(struct script_info *info,
 					 enum uftrace_pattern_type ptype)
 {
