@@ -16,12 +16,12 @@
 #include "utils/list.h"
 #include "utils/utils.h"
 #include "utils/script-python.h"
-
+#include "utils/script-luajit.h"
 
 /* This will be set by getenv("UFTRACE_SCRIPT"). */
 char *script_str;
 
-static enum script_type_t script_lang;
+enum script_type_t script_lang;
 
 /* The below functions are used both in record time and script command. */
 script_uftrace_entry_t script_uftrace_entry;
@@ -36,7 +36,7 @@ struct script_filter_item {
 
 static LIST_HEAD(filters);
 
-static enum script_type_t get_script_type(const char *str)
+enum script_type_t get_script_type(const char *str)
 {
 	char *ext = strrchr(str, '.');
 
@@ -46,6 +46,8 @@ static enum script_type_t get_script_type(const char *str)
 	 */
 	if (!strcmp(ext, ".py"))
 		return SCRIPT_PYTHON;
+	else if (!strcmp(ext, ".lua"))
+		return SCRIPT_LUAJIT;
 
 	return SCRIPT_UNKNOWN;
 }
@@ -107,7 +109,13 @@ int script_init(struct script_info *info, enum uftrace_pattern_type ptype)
 	switch (script_lang) {
 	case SCRIPT_PYTHON:
 		if (script_init_for_python(info, ptype) < 0) {
-			pr_dbg("failed to init python scripting\n");
+			pr_warn("failed to init python scripting\n");
+			script_pathname = NULL;
+		}
+		break;
+	case SCRIPT_LUAJIT:
+		if (script_init_for_luajit(info, ptype) < 0) {
+			pr_warn("failed to init luajit scripting\n");
 			script_pathname = NULL;
 		}
 		break;
@@ -129,6 +137,9 @@ void script_finish(void)
 	case SCRIPT_PYTHON:
 		script_finish_for_python();
 		break;
+	case SCRIPT_LUAJIT:
+		script_finish_for_luajit();
+		break;
 	default:
 		break;
 	}
