@@ -31,6 +31,7 @@ void read_session_map(char *dirname, struct symtabs *symtabs, char *sid)
 	char buf[PATH_MAX];
 	const char *last_libname = NULL;
 	struct uftrace_mmap **maps = &symtabs->maps;
+	struct uftrace_mmap *last_map = NULL;
 
 	snprintf(buf, sizeof(buf), "%s/sid-%.16s.map", dirname, sid);
 	fp = fopen(buf, "rb");
@@ -49,12 +50,6 @@ void read_session_map(char *dirname, struct symtabs *symtabs, char *sid)
 			   &start, &end, prot, path) != 4)
 			continue;
 
-		/* set base address of main executable */
-		if (symtabs->exec_base == 0 && symtabs->filename &&
-		    !strcmp(path, symtabs->filename)) {
-			symtabs->exec_base = start;
-		}
-
 		/* skip the [stack] mapping */
 		if (path[0] == '[') {
 			if (strncmp(path, "[stack", 6) == 0)
@@ -64,6 +59,8 @@ void read_session_map(char *dirname, struct symtabs *symtabs, char *sid)
 
 		/* use first mapping only (even if it's non-exec) */
 		if (last_libname && !strcmp(last_libname, path)) {
+			/* extend last_map to have all segments */
+			last_map->end = end;
 			continue;
 		}
 
@@ -78,7 +75,15 @@ void read_session_map(char *dirname, struct symtabs *symtabs, char *sid)
 		memcpy(map->prot, prot, 4);
 		memcpy(map->libname, path, namelen);
 		map->libname[strlen(path)] = '\0';
+
+		/* set mapping of main executable */
+		if (symtabs->exec_map == NULL && symtabs->filename &&
+		    !strcmp(path, symtabs->filename)) {
+			symtabs->exec_map = map;
+		}
+
 		last_libname = map->libname;
+		last_map = map;
 
 		*maps = map;
 		maps = &map->next;
@@ -105,6 +110,7 @@ void delete_session_map(struct symtabs *symtabs)
 	}
 
 	symtabs->maps = NULL;
+	symtabs->exec_map = NULL;
 }
 
 /**
@@ -439,6 +445,7 @@ void create_task(struct uftrace_session_link *sessions,
 		 struct uftrace_msg_task *msg, bool fork)
 {
 	struct uftrace_task *t;
+	struct uftrace_task *pt;
 	struct uftrace_session *s;
 	struct rb_node *parent = NULL;
 	struct rb_node **p = &sessions->tasks.rb_node;
@@ -466,18 +473,22 @@ void create_task(struct uftrace_session_link *sessions,
 	t->pid = fork ? msg->tid : msg->pid;
 	t->tid = msg->tid;
 	t->ppid = fork ? msg->pid : 0;
+	t->time.stamp = msg->time;
+
+	INIT_LIST_HEAD(&t->children);
+	INIT_LIST_HEAD(&t->siblings);
+
+	pt = find_task(sessions, msg->pid);
+	if (pt != NULL)
+		list_add_tail(&t->siblings, &pt->children);
 
 	s = find_session(sessions, msg->pid, msg->time);
 	if (s == NULL) {
-		struct uftrace_task *parent;
-
-		parent = find_task(sessions, msg->pid);
-		if (parent && parent->sref_last &&
-		    parent->sref_last->start < msg->time)
-			s = parent->sref_last->sess;
+		if (pt && pt->sref_last && pt->sref_last->start < msg->time)
+			s = pt->sref_last->sess;
 	}
 
-	if (s) {
+	if (s != NULL) {
 		add_session_ref(t, s, msg->time);
 		strncpy(t->comm, basename(s->exename), sizeof(t->comm));
 		t->comm[sizeof(t->comm) - 1] = '\0';
@@ -487,6 +498,9 @@ void create_task(struct uftrace_session_link *sessions,
 		t->tid, sizeof(t->comm), s ? t->comm : "unknown",
 		s ? s->sid : "unknown");
 
+	if (sessions->first_task == NULL)
+		sessions->first_task = t;
+
 	rb_link_node(&t->node, parent, p);
 	rb_insert_color(&t->node, &sessions->tasks);
 }
