@@ -774,9 +774,9 @@ enum uftrace_trace_type check_trace_functions(const char *filename)
 	enum uftrace_trace_type ret = TRACE_ERROR;
 	const char *trace_funcs[] = {
 		"__cyg_profile_func_enter",
+		"__fentry__",
 		"mcount",
 		"_mcount",
-		"__fentry__",
 		"__gnu_mcount_nc",
 	};
 	char *name;
@@ -813,7 +813,12 @@ enum uftrace_trace_type check_trace_functions(const char *filename)
 
 		for (i = 0; i < ARRAY_SIZE(trace_funcs); i++) {
 			if (!strcmp(name, trace_funcs[i])) {
-				ret = (i == 0) ? TRACE_CYGPROF : TRACE_MCOUNT;
+				if (i == 0)
+					ret = TRACE_CYGPROF;
+				else if (i == 1)
+					ret = TRACE_FENTRY;
+				else
+					ret = TRACE_MCOUNT;
 				goto out;
 			}
 		}
@@ -873,11 +878,11 @@ static int load_module_symbol_file(struct symtab *symtab, const char *symfile,
 
 		if (*line == '#') {
 			if (!strncmp(line, "# symbols: ", 11)) {
-				addr = strtoull(line + 11, &pos, 10);
-				symtab->nr_alloc = addr;
+				size_t nr_syms = strtoul(line + 11, &pos, 10);
+				size_t size_syms = nr_syms * sizeof(*sym);
 
-				addr *= sizeof(*sym);
-				symtab->sym = xrealloc(symtab->sym, addr);
+				symtab->nr_alloc = nr_syms;
+				symtab->sym = xrealloc(symtab->sym, size_syms);
 			}
 			continue;
 		}
@@ -1232,8 +1237,7 @@ int save_kernel_symbol(char *dirname)
 	while ((len = fread(buf, 1, sizeof(buf), ifp)) > 0)
 		fwrite(buf, 1, len, ofp);
 
-	if (len < 0)
-		ret = len;
+	ret = ferror(ifp) ? -1 : 0;
 
 	fclose(ifp);
 	fclose(ofp);
@@ -1573,6 +1577,7 @@ TEST_CASE(symbol_load_module) {
 		TEST_STREQ(sym->name, stab.sym[i].name);
 	}
 
+	unload_symtab(&test);
 	unlink(symfile);
 	return TEST_OK;
 }
@@ -1592,15 +1597,20 @@ static int add_map(struct dl_phdr_info *info, size_t sz, void *data)
 	for (i = 0; i < info->dlpi_phnum; i++) {
 		if (info->dlpi_phdr[i].p_type != PT_LOAD)
 			continue;
-		map->start = info->dlpi_addr + info->dlpi_phdr[i].p_vaddr;
-		map->end = map->start + info->dlpi_phdr[i].p_memsz;
-		break;
+
+		if (map->start == 0)
+			map->start = info->dlpi_addr + info->dlpi_phdr[i].p_vaddr;
+
+		/* use last PT_LOAD segment for end address */
+		map->end = info->dlpi_addr + info->dlpi_phdr[i].p_vaddr +
+			info->dlpi_phdr[i].p_memsz;
 	}
 
 	map->len   = strlen(exename);
 	strcpy(map->libname, exename);
 
 	symtabs->maps = map;
+	symtabs->exec_map = map;
 	return 1;
 }
 
@@ -1637,6 +1647,7 @@ TEST_CASE(symbol_load_map) {
 	unload_module_symtabs();
 	TEST_EQ(RB_EMPTY_ROOT(&modules), true);
 
+	free(map);
 	return TEST_OK;
 }
 
