@@ -484,16 +484,15 @@ uint64_t parse_time(char *arg, int limited_digits)
 uint64_t parse_timestamp(char *arg)
 {
 	char *sep;
-	unsigned long tmp;
-	uint64_t ts;
+	uint64_t ts, tmp;
 	int len;
 
-	tmp = strtoul(arg, &sep, 0);
+	tmp = strtoull(arg, &sep, 0);
 	ts = tmp * NSEC_PER_SEC;
 
 	if (*sep == '.') {
 		arg = sep + 1;
-		tmp = strtoul(arg, &sep, 0);
+		tmp = strtoull(arg, &sep, 0);
 
 		len = 0;
 		while (isdigit(*arg)) {
@@ -681,8 +680,8 @@ void strv_free(struct strv *strv)
 #define DQUOTE '"'
 #define QUOTES "\'\""
 
-/* escape quotes with backslash - caller should free the returned string */
-char * strquote(char *str, int *len)
+/* escape double-quote with backslash - caller should free the returned string */
+char * json_quote(char *str, int *len)
 {
 	char *p = str;
 	int quote = 0;
@@ -690,20 +689,16 @@ char * strquote(char *str, int *len)
 	int orig_len = *len;
 
 	/* find number of necessary escape */
-	while ((p = strpbrk(p, QUOTES)) != NULL) {
+	while ((p = strchr(p, DQUOTE)) != NULL) {
 		quote++;
 		p++;
 	}
 
 	p = xmalloc(orig_len + quote + 1);
 
-	/* escape single- and double-quotes */
+	/* escape double-quotes */
 	for (i = k = 0; i < orig_len; i++, k++) {
-		if (str[i] == QUOTE) {
-			p[k++] = '\\';
-			p[k] = QUOTE;
-		}
-		else if (str[i] == DQUOTE) {
+		if (str[i] == DQUOTE) {
 			p[k++] = '\\';
 			p[k] = DQUOTE;
 		}
@@ -729,24 +724,30 @@ static int setargs(char *args, char **argv)
 
 		/* consider quotes and update argv */
 		if (*args == QUOTE) {
+			++args;
+			if (*args == QUOTE )
+				continue;
 			if (argv)
-				argv[count] = ++args;
-			while (*args != QUOTE)
+				argv[count] = args;
+			while (*args && *args != QUOTE)
 				++args;
-			if (argv)
+			if (argv && *args)
 				*args = ' ';
 		}
 		else if (*args == DQUOTE) {
+			++args;
+			if (*args == DQUOTE)
+				continue;
 			if (argv)
-				argv[count] = ++args;
-			while (*args != DQUOTE)
+				argv[count] = args;
+			while (*args && *args != DQUOTE)
 				++args;
-			if (argv)
+			if (argv && *args)
 				*args = ' ';
 		}
 		else if (*args == '#') {
 			/* ignore comment line */
-			while (*args != '\n' || *args == '\0')
+			while (*args != '\n' && *args != '\0')
 				++args;
 			continue;
 		}
