@@ -230,9 +230,15 @@ extern void setup_signal(void);
 		fptr(__VA_ARGS__);					\
 })
 
-#define htonq(x)  htobe64(x)
-#define ntohq(x)  be64toh(x)
+#define stringify(s)    __stringify(s)
+#define __stringify(s)  #s
 
+#ifndef htonq
+# define htonq(x)  htobe64(x)
+#endif
+#ifndef ntohq
+# define ntohq(x)  be64toh(x)
+#endif
 /* this comes from /usr/include/elf.h */
 #ifndef ELFDATA2LSB
 # define ELFDATA2LSB	1		/* 2's complement, little endian */
@@ -320,7 +326,7 @@ char *read_exename(void);
 void print_time_unit(uint64_t delta_nsec);
 void print_diff_percent(uint64_t base_nsec, uint64_t delta_nsec);
 void print_diff_time_unit(uint64_t base_nsec, uint64_t pair_nsec);
-void print_diff_count(unsigned long base, unsigned long pair);
+void print_diff_count(uint64_t base, uint64_t pair);
 
 char *setup_pager(void);
 void start_pager(char *pager);
@@ -331,7 +337,7 @@ uint64_t parse_time(char *arg, int limited_digits);
 uint64_t parse_timestamp(char *arg);
 
 char * strjoin(char *left, char *right, const char *delim);
-char * strquote(char *str, int *len);
+char * json_quote(char *str, int *len);
 
 /* strv - string vector */
 struct strv {
